@@ -3,8 +3,9 @@ TESTING_INTERPRETER = true
 require "graphql"
 require "jazz"
 require "benchmark/ips"
-require "ruby-prof"
+require "stackprof"
 require "memory_profiler"
+require "graphql/batch"
 
 module GraphQLBenchmark
   QUERY_STRING = GraphQL::Introspection::INTROSPECTION_QUERY
@@ -14,10 +15,12 @@ module GraphQLBenchmark
   BENCHMARK_PATH = File.expand_path("../", __FILE__)
   CARD_SCHEMA = GraphQL::Schema.from_definition(File.read(File.join(BENCHMARK_PATH, "schema.graphql")))
   ABSTRACT_FRAGMENTS = GraphQL.parse(File.read(File.join(BENCHMARK_PATH, "abstract_fragments.graphql")))
-  ABSTRACT_FRAGMENTS_2 = GraphQL.parse(File.read(File.join(BENCHMARK_PATH, "abstract_fragments_2.graphql")))
+  ABSTRACT_FRAGMENTS_2_QUERY_STRING = File.read(File.join(BENCHMARK_PATH, "abstract_fragments_2.graphql"))
+  ABSTRACT_FRAGMENTS_2 = GraphQL.parse(ABSTRACT_FRAGMENTS_2_QUERY_STRING)
 
   BIG_SCHEMA = GraphQL::Schema.from_definition(File.join(BENCHMARK_PATH, "big_schema.graphql"))
-  BIG_QUERY = GraphQL.parse(File.read(File.join(BENCHMARK_PATH, "big_query.graphql")))
+  BIG_QUERY_STRING = File.read(File.join(BENCHMARK_PATH, "big_query.graphql"))
+  BIG_QUERY = GraphQL.parse(BIG_QUERY_STRING)
 
   FIELDS_WILL_MERGE_SCHEMA = GraphQL::Schema.from_definition("type Query { hello: String }")
   FIELDS_WILL_MERGE_QUERY = GraphQL.parse("{ #{Array.new(5000, "hello").join(" ")} }")
@@ -34,26 +37,40 @@ module GraphQLBenchmark
         x.report("validate - abstract fragments 2") { CARD_SCHEMA.validate(ABSTRACT_FRAGMENTS_2) }
         x.report("validate - big query") { BIG_SCHEMA.validate(BIG_QUERY) }
         x.report("validate - fields will merge") { FIELDS_WILL_MERGE_SCHEMA.validate(FIELDS_WILL_MERGE_QUERY) }
+      when "parse"
+        x.report("scan - introspection") { GraphQL.scan(QUERY_STRING) }
+        x.report("parse - introspection") { GraphQL.parse(QUERY_STRING) }
+        x.report("scan - fragments") { GraphQL.scan(ABSTRACT_FRAGMENTS_2_QUERY_STRING) }
+        x.report("parse - fragments") { GraphQL.parse(ABSTRACT_FRAGMENTS_2_QUERY_STRING) }
+        x.report("scan - big query") { GraphQL.scan(BIG_QUERY_STRING) }
+        x.report("parse - big query") { GraphQL.parse(BIG_QUERY_STRING) }
       else
         raise("Unexpected task #{task}")
       end
     end
   end
 
+  def self.validate_memory
+    FIELDS_WILL_MERGE_SCHEMA.validate(FIELDS_WILL_MERGE_QUERY)
+
+    report = MemoryProfiler.report do
+      FIELDS_WILL_MERGE_SCHEMA.validate(FIELDS_WILL_MERGE_QUERY)
+      nil
+    end
+
+    report.pretty_print
+  end
+
   def self.profile
     # Warm up any caches:
     SCHEMA.execute(document: DOCUMENT)
     # CARD_SCHEMA.validate(ABSTRACT_FRAGMENTS)
     res = nil
-    result = RubyProf.profile do
+    result = StackProf.run(mode: :wall) do
       # CARD_SCHEMA.validate(ABSTRACT_FRAGMENTS)
       res = SCHEMA.execute(document: DOCUMENT)
     end
-    # printer = RubyProf::FlatPrinter.new(result)
-    # printer = RubyProf::GraphHtmlPrinter.new(result)
-    printer = RubyProf::FlatPrinterWithLineNumbers.new(result)
-
-    printer.print(STDOUT, {})
+    StackProf::Report.new(result).print_text
   end
 
   # Adapted from https://github.com/rmosolgo/graphql-ruby/issues/861
@@ -66,13 +83,10 @@ module GraphQLBenchmark
       }
     end
 
-    result = RubyProf.profile do
+    result = StackProf.run(mode: :wall) do
       schema.execute(document: document)
     end
-    printer = RubyProf::FlatPrinter.new(result)
-    # printer = RubyProf::GraphHtmlPrinter.new(result)
-    # printer = RubyProf::FlatPrinterWithLineNumbers.new(result)
-    printer.print(STDOUT, {})
+    StackProf::Report.new(result).print_text
 
     report = MemoryProfiler.report do
       schema.execute(document: document)
@@ -121,14 +135,13 @@ module GraphQLBenchmark
 
     class Schema < GraphQL::Schema
       query QueryType
-      use GraphQL::Execution::Interpreter
-      use GraphQL::Analysis::AST
+      use GraphQL::Dataloader
     end
 
     ALL_FIELDS = GraphQL.parse <<-GRAPHQL
-      {
+      query($skip: Boolean = false) {
         foos {
-          id
+          id @skip(if: $skip)
           int1
           int2
           string1
@@ -142,4 +155,71 @@ module GraphQLBenchmark
       }
     GRAPHQL
   end
+
+  def self.profile_batch_loaders
+    require_relative "./batch_loading"
+    include BatchLoading
+
+    document = GraphQL.parse <<-GRAPHQL
+    {
+      braves: team(name: "Braves") { ...TeamFields }
+      bulls: team(name: "Bulls") { ...TeamFields }
+    }
+
+    fragment TeamFields on Team {
+      players {
+        team {
+          players {
+            team {
+              name
+            }
+          }
+        }
+      }
+    }
+    GRAPHQL
+    batch_result = GraphQLBatchSchema.execute(document: document).to_h
+    dataloader_result = GraphQLDataloaderSchema.execute(document: document).to_h
+    no_batch_result = GraphQLNoBatchingSchema.execute(document: document).to_h
+
+    results = [batch_result, dataloader_result, no_batch_result].uniq
+    if results.size > 1
+      puts "Batch result:"
+      pp batch_result
+      puts "Dataloader result:"
+      pp dataloader_result
+      puts "No-batch result:"
+      pp no_batch_result
+      raise "Got different results -- fix implementation before benchmarking."
+    end
+
+    Benchmark.ips do |x|
+      x.report("GraphQL::Batch") { GraphQLBatchSchema.execute(document: document) }
+      x.report("GraphQL::Dataloader") { GraphQLDataloaderSchema.execute(document: document) }
+      x.report("No Batching") { GraphQLNoBatchingSchema.execute(document: document) }
+
+      x.compare!
+    end
+
+    puts "========== GraphQL-Batch Memory =============="
+    report = MemoryProfiler.report do
+      GraphQLBatchSchema.execute(document: document)
+    end
+
+    report.pretty_print
+
+    puts "========== Dataloader Memory ================="
+    report = MemoryProfiler.report do
+      GraphQLDataloaderSchema.execute(document: document)
+    end
+
+    report.pretty_print
+
+    puts "========== No Batch Memory =============="
+    report = MemoryProfiler.report do
+      GraphQLNoBatchingSchema.execute(document: document)
+    end
+
+    report.pretty_print
+  end
 end
