@@ -36,7 +36,7 @@
 //     // Do something with `data` and/or `errors`
 //   }})
 //
-import { ApolloLink, Observable, FetchResult, NextLink, Operation } from "apollo-link"
+import { ApolloLink, Observable, FetchResult, NextLink, Operation } from "@apollo/client/core"
 import { Realtime } from "ably"
 
 type RequestResult = Observable<FetchResult<{ [key: string]: any; }, Record<string, any>, Record<string, any>>>
