@@ -1,5 +1,5 @@
 import { createActionCableHandler } from "../createActionCableHandler"
-import { Cable } from "actioncable"
+import type { Consumer } from "@rails/actioncable"
 describe("createActionCableHandler", () => {
   it("returns a function producing a disposable subscription", () => {
     var wasDisposed = false
@@ -14,7 +14,7 @@ describe("createActionCableHandler", () => {
     }
 
     var options = {
-      cable: (dummyActionCableConsumer as unknown) as Cable
+      cable: (dummyActionCableConsumer as unknown) as Consumer
     }
     var producer = createActionCableHandler(options)
     producer({text: "", name: ""}, {}, {}, { onError: () => {}, onNext: () => {}, onCompleted: () => {} }).dispose()
