@@ -4,7 +4,6 @@ require "json"
 require "set"
 require "singleton"
 require "forwardable"
-require_relative "./graphql/railtie" if defined? Rails::Railtie
 
 module GraphQL
   # forwards-compat for argument handling
@@ -18,6 +17,17 @@ module GraphQL
   class Error < StandardError
   end
 
+  # This error is raised when GraphQL-Ruby encounters a situation
+  # that it *thought* would never happen. Please report this bug!
+  class InvariantError < Error
+    def initialize(message)
+      message += "
+
+This is probably a bug in GraphQL-Ruby, please report this error on GitHub: https://github.com/rmosolgo/graphql-ruby/issues/new?template=bug_report.md"
+      super(message)
+    end
+  end
+
   class RequiredImplementationMissingError < Error
   end
 
@@ -56,36 +66,26 @@ module GraphQL
   def self.scan_with_ragel(graphql_string)
     GraphQL::Language::Lexer.tokenize(graphql_string)
   end
-
-  # Support Ruby 2.2 by implementing `-"str"`. If we drop 2.2 support, we can remove this backport.
-  module StringDedupBackport
-    refine String do
-      def -@
-        if frozen?
-          self
-        else
-          self.dup.freeze
-        end
-      end
-    end
-  end
-
-  module StringMatchBackport
-    refine String do
-      def match?(pattern)
-        self =~ pattern
-      end
-    end
-  end
 end
 
 # Order matters for these:
 
 require "graphql/execution_error"
+require "graphql/runtime_type_error"
+require "graphql/unresolved_type_error"
+require "graphql/invalid_null_error"
+require "graphql/analysis_error"
+require "graphql/coercion_error"
+require "graphql/invalid_name_error"
+require "graphql/integer_decoding_error"
+require "graphql/integer_encoding_error"
+require "graphql/string_encoding_error"
+require "graphql/date_encoding_error"
+
 require "graphql/define"
+require "graphql/deprecation"
 require "graphql/base_type"
 require "graphql/object_type"
-
 require "graphql/enum_type"
 require "graphql/input_object_type"
 require "graphql/interface_type"
@@ -103,19 +103,23 @@ require "graphql/scalar_type"
 require "graphql/name_validator"
 
 require "graphql/language"
+
+require_relative "./graphql/railtie" if defined? Rails::Railtie
+
 require "graphql/analysis"
 require "graphql/tracing"
 require "graphql/dig"
 require "graphql/execution"
-require "graphql/runtime_type_error"
-require "graphql/unresolved_type_error"
-require "graphql/invalid_null_error"
+require "graphql/pagination"
 require "graphql/schema"
 require "graphql/query"
-require "graphql/directive"
-require "graphql/execution"
 require "graphql/types"
-require "graphql/relay"
+require "graphql/dataloader"
+require "graphql/filter"
+require "graphql/internal_representation"
+require "graphql/directive"
+require "graphql/static_validation"
+require "graphql/execution"
 require "graphql/boolean_type"
 require "graphql/float_type"
 require "graphql/id_type"
@@ -125,19 +129,11 @@ require "graphql/schema/built_in_types"
 require "graphql/schema/loader"
 require "graphql/schema/printer"
 require "graphql/introspection"
+require "graphql/relay"
 
-require "graphql/analysis_error"
-require "graphql/coercion_error"
-require "graphql/invalid_name_error"
-require "graphql/integer_decoding_error"
-require "graphql/integer_encoding_error"
-require "graphql/string_encoding_error"
-require "graphql/internal_representation"
-require "graphql/static_validation"
 require "graphql/version"
 require "graphql/compatibility"
 require "graphql/function"
-require "graphql/filter"
 require "graphql/subscriptions"
 require "graphql/parse_error"
 require "graphql/backtrace"
@@ -147,4 +143,38 @@ require "graphql/authorization"
 require "graphql/unauthorized_error"
 require "graphql/unauthorized_field_error"
 require "graphql/load_application_object_failed_error"
-require "graphql/pagination"
+require "graphql/directive/include_directive"
+require "graphql/directive/skip_directive"
+require "graphql/directive/deprecated_directive"
+
+module GraphQL
+  # Ruby has `deprecate_constant`,
+  # but I don't see a way to give a nice error message in that case,
+  # so I'm doing this instead.
+  DEPRECATED_INT_TYPE = INT_TYPE
+  DEPRECATED_FLOAT_TYPE = FLOAT_TYPE
+  DEPRECATED_STRING_TYPE = STRING_TYPE
+  DEPRECATED_BOOLEAN_TYPE = BOOLEAN_TYPE
+  DEPRECATED_ID_TYPE = ID_TYPE
+
+  remove_const :INT_TYPE
+  remove_const :FLOAT_TYPE
+  remove_const :STRING_TYPE
+  remove_const :BOOLEAN_TYPE
+  remove_const :ID_TYPE
+
+  def self.const_missing(const_name)
+    deprecated_const_name = :"DEPRECATED_#{const_name}"
+    if const_defined?(deprecated_const_name)
+      deprecated_type = const_get(deprecated_const_name)
+      deprecated_caller = caller(1, 1).first
+      # Don't warn about internal uses, like `types.Int`
+      if !deprecated_caller.include?("lib/graphql")
+        warn "GraphQL::#{const_name} is deprecated and will be removed in GraphQL-Ruby 2.0, use GraphQL::Types::#{deprecated_type.graphql_name} instead. (from #{deprecated_caller})"
+      end
+      deprecated_type
+    else
+      super
+    end
+  end
+end
