@@ -133,8 +133,6 @@ module GraphQL
         end
 
         class << self
-          # rubocop:disable Development/NoEvalCop This eval takes static inputs at load-time
-
           # Add a default `#visit_method` and `#children_method_name` using the class name
           def inherited(child_class)
             super
@@ -199,7 +197,16 @@ module GraphQL
               else
                 module_eval <<-RUBY, __FILE__, __LINE__
                   def children
-                    @children ||= (#{children_of_type.keys.map { |k| "@#{k}" }.join(" + ")}).freeze
+                    @children ||= begin
+                      if #{children_of_type.keys.map { |k| "@#{k}.any?" }.join(" || ")}
+                        new_children = []
+                        #{children_of_type.keys.map { |k| "new_children.concat(@#{k})" }.join("; ")}
+                        new_children.freeze
+                        new_children
+                      else
+                        NO_CHILDREN
+                      end
+                    end
                   end
                 RUBY
               end
@@ -267,11 +274,8 @@ module GraphQL
                   #{assignments.join("\n")}
                 end
               RUBY
-
-              # rubocop:enable Development/NoEvalCop
             end
           end
-          # rubocop:enable Development/NoEvalCop
         end
       end
 
@@ -314,7 +318,7 @@ module GraphQL
       class DirectiveDefinition < AbstractNode
         include DefinitionNode
         attr_reader :description
-        scalar_methods :name
+        scalar_methods :name, :repeatable
         children_methods(
           locations: Nodes::DirectiveLocation,
           arguments: Nodes::Argument,
@@ -337,7 +341,7 @@ module GraphQL
       #    end
       #  end
       #
-      #  document.to_query_string(printer: VariableSrubber.new)
+      #  document.to_query_string(printer: VariableScrubber.new)
       #
       class Document < AbstractNode
         scalar_methods false
@@ -472,7 +476,6 @@ module GraphQL
         end
       end
 
-
       # A list type definition, denoted with `[...]` (used for variable type definitions)
       class ListType < WrapperType
       end
@@ -623,6 +626,7 @@ module GraphQL
         attr_reader :description
         scalar_methods :name
         children_methods({
+          interfaces: GraphQL::Language::Nodes::TypeName,
           directives: GraphQL::Language::Nodes::Directive,
           fields: GraphQL::Language::Nodes::FieldDefinition,
         })
@@ -632,6 +636,7 @@ module GraphQL
       class InterfaceTypeExtension < AbstractNode
         scalar_methods :name
         children_methods({
+          interfaces: GraphQL::Language::Nodes::TypeName,
           directives: GraphQL::Language::Nodes::Directive,
           fields: GraphQL::Language::Nodes::FieldDefinition,
         })
