@@ -79,7 +79,7 @@ module GraphQL
 
         arg_owner = @current_input_type || @current_directive || @current_field
         old_current_argument = @current_argument
-        @current_argument = arg_owner.arguments[argument.name]
+        @current_argument = arg_owner.get_argument(argument.name, @query.context)
 
         old_input_type = @current_input_type
         @current_input_type = @current_argument.type.non_null? ? @current_argument.type.of_type : @current_argument.type
@@ -113,7 +113,7 @@ module GraphQL
       end
 
       def print_field(field, indent: "")
-        @current_field = query.schema.get_field(@current_type, field.name)
+        @current_field = query.get_field(@current_type, field.name)
         old_type = @current_type
         @current_type = @current_field.type.unwrap
         res = super
@@ -125,7 +125,7 @@ module GraphQL
         old_type = @current_type
 
         if inline_fragment.type
-          @current_type = query.schema.types[inline_fragment.type.name]
+          @current_type = query.get_type(inline_fragment.type.name)
         end
 
         res = super
@@ -137,7 +137,7 @@ module GraphQL
 
       def print_fragment_definition(fragment_def, indent: "")
         old_type = @current_type
-        @current_type = query.schema.types[fragment_def.type.name]
+        @current_type = query.get_type(fragment_def.type.name)
 
         res = super
 
@@ -193,7 +193,7 @@ module GraphQL
           end
 
           arguments = value.map do |key, val|
-            sub_type = type.arguments[key.to_s].type
+            sub_type = type.get_argument(key.to_s, @query.context).type
 
             GraphQL::Language::Nodes::Argument.new(
               name: key.to_s,
