@@ -4,10 +4,6 @@ require "graphql/schema/build_from_definition/resolve_map"
 module GraphQL
   class Schema
     module BuildFromDefinition
-      if !String.method_defined?(:-@)
-        using GraphQL::StringDedupBackport
-      end
-
       class << self
         # @see {Schema.from_definition}
         def from_definition(definition_string, parser: GraphQL.default_parser, **kwargs)
@@ -18,8 +14,8 @@ module GraphQL
           from_document(parser.parse_file(definition_path), **kwargs)
         end
 
-        def from_document(document, default_resolve:, using: {}, relay: false, interpreter: true)
-          Builder.build(document, default_resolve: default_resolve || {}, relay: relay, using: using, interpreter: interpreter)
+        def from_document(document, default_resolve:, using: {}, relay: false)
+          Builder.build(document, default_resolve: default_resolve || {}, relay: relay, using: using)
         end
       end
 
@@ -27,7 +23,7 @@ module GraphQL
       module Builder
         extend self
 
-        def build(document, default_resolve:, using: {}, interpreter: true, relay:)
+        def build(document, default_resolve:, using: {}, relay:)
           raise InvalidDocumentError.new('Must provide a document ast.') if !document || !document.is_a?(GraphQL::Language::Nodes::Document)
 
           if default_resolve.is_a?(Hash)
@@ -41,46 +37,50 @@ module GraphQL
           schema_definition = schema_defns.first
           types = {}
           directives = {}
-          type_resolver = ->(type) { resolve_type(types, type)  }
+          type_resolver = build_resolve_type(types, directives, ->(type_name) { types[type_name] ||= Schema::LateBoundType.new(type_name)})
+          # Make a different type resolver because we need to coerce directive arguments
+          # _while_ building the schema.
+          # It will dig for a type if it encounters a custom type. This could be a problem if there are cycles.
+          directive_type_resolver = nil
+          directive_type_resolver = build_resolve_type(types, directives, ->(type_name) {
+            types[type_name] ||= begin
+              defn = document.definitions.find { |d| d.respond_to?(:name) && d.name == type_name }
+              if defn
+                build_definition_from_node(defn, directive_type_resolver, default_resolve)
+              elsif (built_in_defn = GraphQL::Schema::BUILT_IN_TYPES[type_name])
+                built_in_defn
+              else
+                raise "No definition for #{type_name.inspect} found in schema document or built-in types. Add a definition for it or remove it."
+              end
+            end
+          })
 
           document.definitions.each do |definition|
-            case definition
-            when GraphQL::Language::Nodes::SchemaDefinition
-              nil # already handled
-            when GraphQL::Language::Nodes::EnumTypeDefinition
-              types[definition.name] = build_enum_type(definition, type_resolver)
-            when GraphQL::Language::Nodes::ObjectTypeDefinition
-              types[definition.name] = build_object_type(definition, type_resolver)
-            when GraphQL::Language::Nodes::InterfaceTypeDefinition
-              types[definition.name] = build_interface_type(definition, type_resolver)
-            when GraphQL::Language::Nodes::UnionTypeDefinition
-              types[definition.name] = build_union_type(definition, type_resolver)
-            when GraphQL::Language::Nodes::ScalarTypeDefinition
-              types[definition.name] = build_scalar_type(definition, type_resolver, default_resolve: default_resolve)
-            when GraphQL::Language::Nodes::InputObjectTypeDefinition
-              types[definition.name] = build_input_object_type(definition, type_resolver)
-            when GraphQL::Language::Nodes::DirectiveDefinition
-              directives[definition.name] = build_directive(definition, type_resolver)
-            end
-          end
-
-          # At this point, if types named by the built in types are _late-bound_ types,
-          # that means they were referenced in the schema but not defined in the schema.
-          # That's supported for built-in types. (Eg, you can use `String` without defining it.)
-          # In that case, insert the concrete type definition now.
-          #
-          # However, if the type in `types` is a _concrete_ type definition, that means that
-          # the document contained an explicit definition of the scalar type.
-          # Don't override it in this case.
-          GraphQL::Schema::BUILT_IN_TYPES.each do |scalar_name, built_in_scalar|
-            existing_type = types[scalar_name]
-            if existing_type.is_a?(GraphQL::Schema::LateBoundType)
-              types[scalar_name] = built_in_scalar
+            if definition.is_a?(GraphQL::Language::Nodes::DirectiveDefinition)
+              directives[definition.name] = build_directive(definition, directive_type_resolver)
             end
           end
 
           directives = GraphQL::Schema.default_directives.merge(directives)
 
+          # In case any directives referenced built-in types for their arguments:
+          replace_late_bound_types_with_built_in(types)
+
+          document.definitions.each do |definition|
+            case definition
+            when GraphQL::Language::Nodes::SchemaDefinition, GraphQL::Language::Nodes::DirectiveDefinition
+              nil # already handled
+            else
+              # It's possible that this was already loaded by the directives
+              prev_type = types[definition.name]
+              if prev_type.nil? || prev_type.is_a?(Schema::LateBoundType)
+                types[definition.name] = build_definition_from_node(definition, type_resolver, default_resolve)
+              end
+            end
+          end
+
+          replace_late_bound_types_with_built_in(types)
+
           if schema_definition
             if schema_definition.query
               raise InvalidDocumentError.new("Specified query type \"#{schema_definition.query}\" not found in document.") unless types[schema_definition.query]
@@ -133,11 +133,6 @@ module GraphQL
               ast_node(schema_definition)
             end
 
-            if interpreter
-              use GraphQL::Execution::Interpreter
-              use GraphQL::Analysis::AST
-            end
-
             using.each do |plugin, options|
               if options
                 use(plugin, **options)
@@ -169,10 +164,85 @@ module GraphQL
           raise(GraphQL::RequiredImplementationMissingError, "Generated Schema cannot use Interface or Union types for execution. Implement resolve_type on your resolver.")
         }
 
+        def build_definition_from_node(definition, type_resolver, default_resolve)
+          case definition
+          when GraphQL::Language::Nodes::EnumTypeDefinition
+            build_enum_type(definition, type_resolver)
+          when GraphQL::Language::Nodes::ObjectTypeDefinition
+            build_object_type(definition, type_resolver)
+          when GraphQL::Language::Nodes::InterfaceTypeDefinition
+            build_interface_type(definition, type_resolver)
+          when GraphQL::Language::Nodes::UnionTypeDefinition
+            build_union_type(definition, type_resolver)
+          when GraphQL::Language::Nodes::ScalarTypeDefinition
+            build_scalar_type(definition, type_resolver, default_resolve: default_resolve)
+          when GraphQL::Language::Nodes::InputObjectTypeDefinition
+            build_input_object_type(definition, type_resolver)
+          end
+        end
+
+        # Modify `types`, replacing any late-bound references to built-in types
+        # with their actual definitions.
+        #
+        # (Schema definitions are allowed to reference those built-ins without redefining them.)
+        # @return void
+        def replace_late_bound_types_with_built_in(types)
+          GraphQL::Schema::BUILT_IN_TYPES.each do |scalar_name, built_in_scalar|
+            existing_type = types[scalar_name]
+            if existing_type.is_a?(GraphQL::Schema::LateBoundType)
+              types[scalar_name] = built_in_scalar
+            end
+          end
+        end
+
+        def build_directives(definition, ast_node, type_resolver)
+          dirs = prepare_directives(ast_node, type_resolver)
+          dirs.each do |dir_class, options|
+            definition.directive(dir_class, **options)
+          end
+        end
+
+        def prepare_directives(ast_node, type_resolver)
+          dirs = {}
+          ast_node.directives.each do |dir_node|
+            if dir_node.name == "deprecated"
+              # This is handled using `deprecation_reason`
+              next
+            else
+              dir_class = type_resolver.call(dir_node.name)
+              if dir_class.nil?
+                raise ArgumentError, "No definition for @#{dir_node.name} on #{ast_node.name} at #{ast_node.line}:#{ast_node.col}"
+              end
+              options = args_to_kwargs(dir_class, dir_node)
+              dirs[dir_class] = options
+            end
+          end
+          dirs
+        end
+
+        def args_to_kwargs(arg_owner, node)
+          if node.respond_to?(:arguments)
+            kwargs = {}
+            node.arguments.each do |arg_node|
+              arg_defn = arg_owner.get_argument(arg_node.name)
+              kwargs[arg_defn.keyword] = args_to_kwargs(arg_defn.type.unwrap, arg_node.value)
+            end
+            kwargs
+          elsif node.is_a?(Array)
+            node.map { |n| args_to_kwargs(arg_owner, n) }
+          elsif node.is_a?(Language::Nodes::Enum)
+            node.name
+          else
+            # scalar
+            node
+          end
+        end
+
         def build_enum_type(enum_type_definition, type_resolver)
           builder = self
           Class.new(GraphQL::Schema::Enum) do
             graphql_name(enum_type_definition.name)
+            builder.build_directives(self, enum_type_definition, type_resolver)
             description(enum_type_definition.description)
             ast_node(enum_type_definition)
             enum_type_definition.values.each do |enum_value_definition|
@@ -180,6 +250,7 @@ module GraphQL
                 value: enum_value_definition.name,
                 deprecation_reason: builder.build_deprecation_reason(enum_value_definition.directives),
                 description: enum_value_definition.description,
+                directives: builder.prepare_directives(enum_value_definition, type_resolver),
                 ast_node: enum_value_definition,
               )
             end
@@ -202,6 +273,7 @@ module GraphQL
             graphql_name(scalar_type_definition.name)
             description(scalar_type_definition.description)
             ast_node(scalar_type_definition)
+            builder.build_directives(self, scalar_type_definition, type_resolver)
 
             if default_resolve.respond_to?(:coerce_input)
               # Put these method definitions in another method to avoid retaining `type_resolve`
@@ -219,11 +291,13 @@ module GraphQL
         end
 
         def build_union_type(union_type_definition, type_resolver)
+          builder = self
           Class.new(GraphQL::Schema::Union) do
             graphql_name(union_type_definition.name)
             description(union_type_definition.description)
             possible_types(*union_type_definition.types.map { |type_name| type_resolver.call(type_name) })
             ast_node(union_type_definition)
+            builder.build_directives(self, union_type_definition, type_resolver)
           end
         end
 
@@ -234,6 +308,7 @@ module GraphQL
             graphql_name(object_type_definition.name)
             description(object_type_definition.description)
             ast_node(object_type_definition)
+            builder.build_directives(self, object_type_definition, type_resolver)
 
             object_type_definition.interfaces.each do |interface_name|
               interface_defn = type_resolver.call(interface_name)
@@ -250,6 +325,7 @@ module GraphQL
             graphql_name(input_object_type_definition.name)
             description(input_object_type_definition.description)
             ast_node(input_object_type_definition)
+            builder.build_directives(self, input_object_type_definition, type_resolver)
             builder.build_arguments(self, input_object_type_definition.fields, type_resolver)
           end
         end
@@ -290,6 +366,7 @@ module GraphQL
               ast_node: argument_defn,
               camelize: false,
               method_access: false,
+              directives: prepare_directives(argument_defn, type_resolver),
               **default_value_kwargs
             )
           end
@@ -300,6 +377,7 @@ module GraphQL
           Class.new(GraphQL::Schema::Directive) do
             graphql_name(directive_definition.name)
             description(directive_definition.description)
+            repeatable(directive_definition.repeatable)
             locations(*directive_definition.locations.map { |location| location.name.to_sym })
             ast_node(directive_definition)
             builder.build_arguments(self, directive_definition.arguments, type_resolver)
@@ -312,7 +390,13 @@ module GraphQL
             include GraphQL::Schema::Interface
             graphql_name(interface_type_definition.name)
             description(interface_type_definition.description)
+            interface_type_definition.interfaces.each do |interface_name|
+              "Implements: #{interface_type_definition} -> #{interface_name}"
+              interface_defn = type_resolver.call(interface_name)
+              implements(interface_defn)
+            end
             ast_node(interface_type_definition)
+            builder.build_directives(self, interface_type_definition, type_resolver)
 
             builder.build_fields(self, interface_type_definition.fields, type_resolver, default_resolve: nil)
           end
@@ -335,6 +419,7 @@ module GraphQL
               ast_node: field_definition,
               method_conflict_warning: false,
               camelize: false,
+              directives: prepare_directives(field_definition, type_resolver),
               resolver_method: resolve_method_name,
             )
 
@@ -342,30 +427,39 @@ module GraphQL
 
             # Don't do this for interfaces
             if default_resolve
-              define_field_resolve_method(owner, resolve_method_name, field_definition.name)
+              owner.class_eval <<-RUBY, __FILE__, __LINE__
+                # frozen_string_literal: true
+                def #{resolve_method_name}(**args)
+                  field_instance = self.class.get_field("#{field_definition.name}")
+                  context.schema.definition_default_resolve.call(self.class, field_instance, object, args, context)
+                end
+              RUBY
             end
           end
         end
 
-        def resolve_type(types, ast_node)
-          case ast_node
-          when GraphQL::Language::Nodes::TypeName
-            type_name = ast_node.name
-            types[type_name] ||= GraphQL::Schema::LateBoundType.new(type_name)
-          when GraphQL::Language::Nodes::NonNullType
-            resolve_type(types, ast_node.of_type).to_non_null_type
-          when GraphQL::Language::Nodes::ListType
-            resolve_type(types, ast_node.of_type).to_list_type
-          else
-            raise "Unexpected ast_node: #{ast_node.inspect}"
-          end
-        end
-
-        def define_field_resolve_method(owner, method_name, field_name)
-          owner.define_method(method_name) { |**args|
-            field_instance = self.class.get_field(field_name)
-            context.schema.definition_default_resolve.call(self.class, field_instance, object, args, context)
+        def build_resolve_type(lookup_hash, directives, missing_type_handler)
+          resolve_type_proc = nil
+          resolve_type_proc = ->(ast_node) {
+            case ast_node
+            when GraphQL::Language::Nodes::TypeName
+              type_name = ast_node.name
+              if lookup_hash.key?(type_name)
+                lookup_hash[type_name]
+              else
+                missing_type_handler.call(type_name)
+              end
+            when GraphQL::Language::Nodes::NonNullType
+              resolve_type_proc.call(ast_node.of_type).to_non_null_type
+            when GraphQL::Language::Nodes::ListType
+              resolve_type_proc.call(ast_node.of_type).to_list_type
+            when String
+              directives[ast_node]
+            else
+              raise "Unexpected ast_node: #{ast_node.inspect}"
+            end
           }
+          resolve_type_proc
         end
 
         def resolve_type_name(type)
