@@ -2,7 +2,7 @@
 module Platform
   module Objects
     class Photo < Platform::Objects::Base
-      field :caption, String, null: true
+      field :caption, String
 
       def caption
         object.caption
