@@ -117,7 +117,7 @@ describe GraphQL::Authorization do
         super && !ctx[:hide]
       end
 
-      field :some_field, String, null: true
+      field :some_field, String
     end
 
     class RelayObject < BaseObject
@@ -133,7 +133,7 @@ describe GraphQL::Authorization do
         super && !ctx[:unauthorized_relay]
       end
 
-      field :some_field, String, null: true
+      field :some_field, String
     end
 
     # TODO test default behavior for abstract types,
@@ -157,7 +157,7 @@ describe GraphQL::Authorization do
         InaccessibleObject
       end
 
-      field :some_field, String, null: true
+      field :some_field, String
     end
 
     class InaccessibleObject < BaseObject
@@ -167,7 +167,7 @@ describe GraphQL::Authorization do
         super && !ctx[:hide]
       end
 
-      field :some_field, String, null: true
+      field :some_field, String
     end
 
     class UnauthorizedObject < BaseObject
@@ -269,8 +269,8 @@ describe GraphQL::Authorization do
       end
 
       field :hidden, Integer, null: false
-      field :unauthorized, Integer, null: true, method: :itself
-      field :int2, Integer, null: true do
+      field :unauthorized, Integer, method: :itself
+      field :int2, Integer do
         argument :int, Integer, required: false
         argument :hidden, Integer, required: false
         argument :inaccessible, Integer, required: false
@@ -313,7 +313,7 @@ describe GraphQL::Authorization do
       field :inaccessible_connection, RelayObject.connection_type, null: :false, resolver_method: :empty_array
       field :inaccessible_edge, RelayObject.edge_type, null: :false, resolver_method: :edge_object
 
-      field :unauthorized_object, UnauthorizedObject, null: true, resolver_method: :itself
+      field :unauthorized_object, UnauthorizedObject, resolver_method: :itself
       field :unauthorized_connection, RelayObject.connection_type, null: false, resolver_method: :array_with_item
       field :unauthorized_edge, RelayObject.edge_type, null: false, resolver_method: :edge_object
 
@@ -325,27 +325,27 @@ describe GraphQL::Authorization do
         [1]
       end
 
-      field :unauthorized_lazy_box, UnauthorizedBox, null: true do
-        argument :value, String, required: true
+      field :unauthorized_lazy_box, UnauthorizedBox do
+        argument :value, String
       end
       def unauthorized_lazy_box(value:)
         # Make it extra nested, just for good measure.
         Box.new(value: Box.new(value: value))
       end
-      field :unauthorized_list_items, [UnauthorizedObject], null: true
+      field :unauthorized_list_items, [UnauthorizedObject]
       def unauthorized_list_items
         [self, self]
       end
 
-      field :unauthorized_lazy_check_box, UnauthorizedCheckBox, null: true, resolver_method: :unauthorized_lazy_box do
-        argument :value, String, required: true
+      field :unauthorized_lazy_check_box, UnauthorizedCheckBox, resolver_method: :unauthorized_lazy_box do
+        argument :value, String
       end
 
-      field :unauthorized_interface, UnauthorizedInterface, null: true, resolver_method: :unauthorized_lazy_box do
-        argument :value, String, required: true
+      field :unauthorized_interface, UnauthorizedInterface, resolver_method: :unauthorized_lazy_box do
+        argument :value, String
       end
 
-      field :unauthorized_lazy_list_interface, [UnauthorizedInterface, null: true], null: true
+      field :unauthorized_lazy_list_interface, [UnauthorizedInterface, null: true]
 
       def unauthorized_lazy_list_interface
         ["z", Box.new(value: Box.new(value: "z2")), "a", Box.new(value: "a")]
@@ -380,7 +380,7 @@ describe GraphQL::Authorization do
         super && !ctx[:hidden_mutation]
       end
 
-      field :some_return_field, String, null: true
+      field :some_return_field, String
     end
 
     class DoInaccessibleStuff < GraphQL::Schema::RelayClassicMutation
@@ -402,17 +402,17 @@ describe GraphQL::Authorization do
       field :do_unauthorized_stuff, mutation: DoUnauthorizedStuff
     end
 
-    class Schema < GraphQL::Schema
-      if TESTING_INTERPRETER
-        use GraphQL::Execution::Interpreter
-        use GraphQL::Analysis::AST
-      else
-        # Opt in to accessible? checks
-        query_analyzer GraphQL::Authorization::Analyzer
+    class Nothing < GraphQL::Schema::Directive
+      locations(FIELD)
+      def self.visible?(ctx)
+        !!ctx[:show_nothing_directive]
       end
+    end
 
+    class Schema < GraphQL::Schema
       query(Query)
       mutation(Mutation)
+      directive(Nothing)
 
       lazy_resolve(Box, :value)
 
@@ -432,10 +432,6 @@ describe GraphQL::Authorization do
     end
 
     class SchemaWithFieldHook < GraphQL::Schema
-      if TESTING_INTERPRETER
-        use GraphQL::Execution::Interpreter
-        use GraphQL::Analysis::AST
-      end
       query(Query)
 
       lazy_resolve(Box, :value)
@@ -560,7 +556,7 @@ describe GraphQL::Authorization do
     end
 
     it "refuses to resolve to hidden enum values" do
-      expected_class = TESTING_INTERPRETER ? AuthTest::LandscapeFeature::UnresolvedValueError : GraphQL::EnumType::UnresolvedValueError
+      expected_class = AuthTest::LandscapeFeature::UnresolvedValueError
       assert_raises(expected_class) do
         auth_execute <<-GRAPHQL, context: { hide: true }
         {
@@ -613,66 +609,13 @@ describe GraphQL::Authorization do
       refute_includes restricted_sdl, 'Hidden'
       refute_includes restricted_sdl, 'hidden'
     end
-  end
 
-  if !TESTING_INTERPRETER
-    # This isn't supported when running the interpreter
-    describe "applying the accessible? method" do
-      it "works with fields and arguments" do
-        queries = {
-          "{ inaccessible }" => ["Some fields in this query are not accessible: inaccessible"],
-          "{ int2(inaccessible: 1) }" => ["Some fields in this query are not accessible: int2"],
-        }
-
-        queries.each do |query_str, errors|
-          res = auth_execute(query_str, context: { hide: true })
-          assert_equal errors, res.fetch("errors").map { |e| e["message"] }
-
-          res = auth_execute(query_str, context: { hide: false })
-          refute res.key?("errors")
-        end
-      end
-
-      it "works with return types" do
-        queries = {
-          "{ inaccessibleObject { __typename } }" => ["Some fields in this query are not accessible: inaccessibleObject"],
-          "{ inaccessibleInterface { __typename } }" => ["Some fields in this query are not accessible: inaccessibleInterface"],
-          "{ inaccessibleDefaultInterface { __typename } }" => ["Some fields in this query are not accessible: inaccessibleDefaultInterface"],
-        }
-
-        queries.each do |query_str, errors|
-          res = auth_execute(query_str, context: { hide: true })
-          assert_equal errors, res["errors"].map { |e| e["message"] }
-
-          res = auth_execute(query_str, context: { hide: false })
-          refute res.key?("errors")
-        end
-      end
-
-      it "works with mutations" do
-        query = "mutation { doInaccessibleStuff(input: {}) { __typename } }"
-        res = auth_execute(query, context: { inaccessible_mutation: true })
-        assert_equal ["Some fields in this query are not accessible: doInaccessibleStuff"], res["errors"].map { |e| e["message"] }
-
-        assert_raises GraphQL::RequiredImplementationMissingError do
-          auth_execute(query)
-        end
-      end
-
-      it "works with edges and connections" do
-        query = <<-GRAPHQL
-        {
-          inaccessibleConnection { __typename }
-          inaccessibleEdge { __typename }
-        }
-        GRAPHQL
-
-        inaccessible_res = auth_execute(query, context: { inaccessible_relay: true })
-        assert_equal ["Some fields in this query are not accessible: inaccessibleConnection, inaccessibleEdge"], inaccessible_res["errors"].map { |e| e["message"] }
-
-        accessible_res = auth_execute(query)
-        refute accessible_res.key?("errors")
-      end
+    it "works with directives" do
+      query_str = "{ __typename @nothing }"
+      visible_response = auth_execute(query_str, context: { show_nothing_directive: true })
+      assert_equal "Query", visible_response["data"]["__typename"]
+      hidden_response = auth_execute(query_str)
+      assert_equal ["Directive @nothing is not defined"], hidden_response["errors"].map { |e| e["message"] }
     end
   end
 
@@ -719,19 +662,18 @@ describe GraphQL::Authorization do
             end
           end
 
-          if TESTING_INTERPRETER
-            describe "when the field authorization resolves lazily" do
-              it "returns value if authorized" do
-                query = "{ unauthorized }"
-                response = AuthTest::SchemaWithFieldHook.execute(query, root_value: 34, context: { lazy_field_authorized: true })
-                assert_equal 34, response["data"].fetch("unauthorized")
-              end
 
-              it "returns nil if not authorized" do
-                query = "{ unauthorized }"
-                response = AuthTest::SchemaWithFieldHook.execute(query, root_value: 34, context: { lazy_field_authorized: false })
-                assert_nil response["data"].fetch("unauthorized")
-              end
+          describe "when the field authorization resolves lazily" do
+            it "returns value if authorized" do
+              query = "{ unauthorized }"
+              response = AuthTest::SchemaWithFieldHook.execute(query, root_value: 34, context: { lazy_field_authorized: true })
+              assert_equal 34, response["data"].fetch("unauthorized")
+            end
+
+            it "returns nil if not authorized" do
+              query = "{ unauthorized }"
+              response = AuthTest::SchemaWithFieldHook.execute(query, root_value: 34, context: { lazy_field_authorized: false })
+              assert_nil response["data"].fetch("unauthorized")
             end
           end
 
@@ -837,7 +779,7 @@ describe GraphQL::Authorization do
       # the same.
       #
       # TODO revisit the docs for this.
-      failed_nodes_value = TESTING_INTERPRETER ? [nil] : nil
+      failed_nodes_value = [nil]
       assert_equal failed_nodes_value, conn.fetch("nodes")
       assert_equal [{"node" => nil, "__typename" => "RelayObjectEdge"}], conn.fetch("edges")
 
@@ -847,7 +789,7 @@ describe GraphQL::Authorization do
 
       unauthorized_object_paths = [
         ["unauthorizedConnection", "edges", 0, "node"],
-        TESTING_INTERPRETER ? ["unauthorizedConnection", "nodes", 0] : ["unauthorizedConnection", "nodes"],
+        ["unauthorizedConnection", "nodes", 0],
         ["unauthorizedEdge", "node"]
       ]
 
@@ -1016,11 +958,6 @@ describe GraphQL::Authorization do
         end
       end
       query(Query)
-
-      if TESTING_INTERPRETER
-        use GraphQL::Execution::Interpreter
-        use GraphQL::Analysis::AST
-      end
     end
 
     it "works out-of-the-box" do
@@ -1029,4 +966,54 @@ describe GraphQL::Authorization do
       refute res.key?("errors")
     end
   end
+
+  describe "overriding authorized_new" do
+    class AuthorizedNewOverrideSchema < GraphQL::Schema
+      class LogTracer
+        def trace(key, data)
+          if (c = data[:context]) || ((q = data[:query]) && (c = q.context))
+            c[:log] << key
+          end
+          yield
+        end
+      end
+
+      module CustomIntrospection
+        class DynamicFields < GraphQL::Introspection::DynamicFields
+          def self.authorized_new(obj, ctx)
+            new(obj, ctx)
+          end
+        end
+      end
+
+      class Query < GraphQL::Schema::Object
+        def self.authorized_new(obj, ctx)
+          new(obj, ctx)
+        end
+        field :int, Integer, null: false
+        def int; 1; end
+      end
+
+      query(Query)
+      introspection(CustomIntrospection)
+      tracer(LogTracer.new)
+    end
+
+    it "avoids calls to Object.authorized?" do
+      log = []
+      res = AuthorizedNewOverrideSchema.execute("{ __typename int }", context: { log: log })
+      assert_equal "Query", res["data"]["__typename"]
+      assert_equal 1, res["data"]["int"]
+      expected_log = [
+        "validate",
+        "analyze_query",
+        "execute_query",
+        "execute_field",
+        "execute_field",
+        "execute_query_lazy"
+      ]
+
+      assert_equal expected_log, log
+    end
+  end
 end
