@@ -8,8 +8,9 @@ describe GraphQL::Introspection::TypeType do
        milkType:      __type(name: "Milk") { interfaces { name }, fields { type { kind, name, ofType { name } } } }
        dairyAnimal:   __type(name: "DairyAnimal") { name, kind, enumValues(includeDeprecated: false) { name, isDeprecated } }
        dairyProduct:  __type(name: "DairyProduct") { name, kind, possibleTypes { name } }
-       animalProduct: __type(name: "AnimalProduct") { name, kind, possibleTypes { name }, fields { name } }
+       animalProduct: __type(name: "AnimalProduct") { name, kind, specifiedByUrl, possibleTypes { name }, fields { name } }
        missingType:   __type(name: "NotAType") { name }
+       timeType:      __type(name: "Time") { specifiedByUrl }
      }
   |}
   let(:result) { Dummy::Schema.execute(query_string, context: {}, variables: {"cheeseId" => 2}) }
@@ -40,9 +41,9 @@ describe GraphQL::Introspection::TypeType do
       },
       "milkType"=>{
         "interfaces"=>[
+          {"name"=>"AnimalProduct"},
           {"name"=>"Edible"},
           {"name"=>"EdibleAsMilk"},
-          {"name"=>"AnimalProduct"},
           {"name"=>"LocalProduct"},
         ],
         "fields"=>[
@@ -69,12 +70,14 @@ describe GraphQL::Introspection::TypeType do
       "animalProduct" => {
         "name"=>"AnimalProduct",
         "kind"=>"INTERFACE",
+        "specifiedByUrl" => nil,
         "possibleTypes"=>[{"name"=>"Cheese"}, {"name"=>"Honey"}, {"name"=>"Milk"}],
         "fields"=>[
           {"name"=>"source"},
         ]
       },
       "missingType" => nil,
+      "timeType" => { "specifiedByUrl" => "https://time.graphql"}
     }}
     assert_equal(expected, result.to_h)
   end
@@ -108,8 +111,8 @@ describe GraphQL::Introspection::TypeType do
     it "hides deprecated field arguments by default" do
       result = Dummy::Schema.execute <<-GRAPHQL
       {
-        __type(name: "Query") { 
-          fields { 
+        __type(name: "Query") {
+          fields {
             name
             args {
               name
@@ -129,8 +132,8 @@ describe GraphQL::Introspection::TypeType do
     it "can expose deprecated field arguments" do
       result = Dummy::Schema.execute <<-GRAPHQL
       {
-        __type(name: "Query") { 
-          fields { 
+        __type(name: "Query") {
+          fields {
             name
             args(includeDeprecated: true) {
               name
@@ -179,8 +182,8 @@ describe GraphQL::Introspection::TypeType do
     it "can expose deprecated input fields" do
       result = Dummy::Schema.execute <<-GRAPHQL
       {
-        __type(name: "DairyProductInput") { 
-          inputFields(includeDeprecated: true) { 
+        __type(name: "DairyProductInput") {
+          inputFields(includeDeprecated: true) {
             name
             isDeprecated
             deprecationReason
@@ -224,7 +227,7 @@ describe GraphQL::Introspection::TypeType do
       field_result = type_result["fields"].find { |f| f["name"] == "bases" }
       all_arg_names = ["after", "before", "first", "last", "nameIncludes", "complexOrder"]
       returned_arg_names = field_result["args"].map { |a| a["name"] }
-      assert_equal all_arg_names, returned_arg_names
+      assert_equal all_arg_names.sort, returned_arg_names.sort
     end
   end
 end
