@@ -59,24 +59,38 @@ describe GraphQL::Language::DocumentFromSchemaDefinition do
 
     let(:expected_document) { GraphQL.parse(expected_idl) }
 
-    describe "when schemas have enums" do
-      let(:schema_idl) { <<-GRAPHQL.chomp
+    describe "when schemas have enums and directives" do
+      let(:schema_idl) { <<-GRAPHQL
 directive @locale(lang: LangEnum!) on FIELD
 
+directive @secret(top: Boolean = false) on FIELD_DEFINITION
+
 enum LangEnum {
   en
   ru
 }
 
 type Query {
-  i: Int
+  i: Int @secret
+  ssn: String @secret(top: true)
 }
       GRAPHQL
       }
 
       class DirectiveSchema < GraphQL::Schema
+        class Secret < GraphQL::Schema::Directive
+          argument :top, Boolean, required: false, default_value: false
+          locations FIELD_DEFINITION
+        end
+
         class Query < GraphQL::Schema::Object
-          field :i, Int, null: true
+          field :i, Int do
+            directive Secret
+          end
+
+          field :ssn, String do
+            directive Secret, top: true
+          end
         end
 
         class Locale < GraphQL::Schema::Directive
@@ -86,7 +100,7 @@ type Query {
           end
           locations GraphQL::Schema::Directive::FIELD
 
-          argument :lang, LangEnum, required: true
+          argument :lang, LangEnum
         end
 
         query(Query)
@@ -813,4 +827,19 @@ type Query {
       expected == node
     end
   end
+
+  describe "custom SDL directives" do
+    class CustomSDLDirectiveSchema < GraphQL::Schema
+      class CustomThing < GraphQL::Schema::Directive
+        locations(FIELD_DEFINITION)
+        argument :stuff, String
+      end
+
+      directive CustomThing
+    end
+
+    it "prints them out" do
+      assert_equal "directive @customThing(stuff: String!) on FIELD_DEFINITION\n", CustomSDLDirectiveSchema.to_definition
+    end
+  end
 end
