@@ -12,51 +12,49 @@ describe GraphQL::Language::SanitizedPrinter do
     end
 
     class SimpleInput < GraphQL::Schema::InputObject
-      argument :string, String, required: true
+      argument :string, String
     end
 
     class ExampleInput < GraphQL::Schema::InputObject
-      argument :string, String, required: true
-      argument :id, ID, required: true
-      argument :int, Int, required: true
-      argument :float, Float, required: true
-      argument :enum, Color, required: true
+      argument :string, String
+      argument :id, ID
+      argument :int, Int
+      argument :float, Float
+      argument :enum, Color
       argument :input_object, ExampleInput, required: false
-      argument :url, Url, required: true
+      argument :url, Url
     end
 
     class Query < GraphQL::Schema::Object
       field :inputs, String, null: false do
-        argument :string, String, required: true
-        argument :id, ID, required: true
-        argument :int, Int, required: true
-        argument :float, Float, required: true
-        argument :enum, Color, required: true
-        argument :input_object, ExampleInput, required: true
-        argument :url, Url, required: true
+        argument :string, String
+        argument :id, ID
+        argument :int, Int
+        argument :float, Float
+        argument :enum, Color
+        argument :input_object, ExampleInput
+        argument :url, Url
       end
 
       field :nested_array_inputs, String, null: false do
-        argument :inputs, [SimpleInput], required: true
+        argument :inputs, [SimpleInput]
       end
 
       field :colors, String, null: false do
-        argument :colors, [Color], required: true
+        argument :colors, [Color]
       end
 
       field :strings, String, null: false do
-        argument :strings, [String], required: true
+        argument :strings, [String]
       end
 
       field :custom_scalar, String, null: false do
-        argument :scalar, GraphQL::Types::JSON, required: true
+        argument :scalar, GraphQL::Types::JSON
       end
     end
 
     class Schema < GraphQL::Schema
       query(Query)
-      use GraphQL::Execution::Interpreter
-      use GraphQL::Analysis::AST
     end
 
     class CustomSanitizedPrinter < GraphQL::Language::SanitizedPrinter
@@ -68,6 +66,11 @@ describe GraphQL::Language::SanitizedPrinter do
         "<#{argument.graphql_name}-redacted>"
       end
     end
+
+    class CustomSanitizedPrinterSchema < GraphQL::Schema
+      query(Query)
+      sanitized_printer(CustomSanitizedPrinter)
+    end
   end
 
   def sanitize_string(query_string, inline_variables: true, **options)
@@ -310,5 +313,36 @@ describe GraphQL::Language::SanitizedPrinter do
     sanitized_query = SanitizeTest::CustomSanitizedPrinter.new(query).sanitized_query_string
     assert_equal expected_query_string, sanitized_query
   end
+
+  it 'configure a custom printer from the schema' do
+    query_str = '
+    {
+      inputs(
+        string: "string",
+        id: "id",
+        int: 1,
+        float: 2.0,
+        url: "http://graphqliscool.com",
+        enum: RED
+        inputObject: {
+          string: "string"
+          id: "id"
+          int: 1
+          float: 2.0
+          url: "http://graphqliscool.com"
+          enum: RED
+        }
+      )
+    }
+    '
+
+    expected_query_string = 'query {
+  inputs(' +
+      'string: <string-redacted>, id: <id-redacted>, int: <int-redacted>, float: <float-redacted>, url: <url-redacted>, enum: RED, inputObject: {' +
+      'string: <string-redacted>, id: <id-redacted>, int: <int-redacted>, float: <float-redacted>, url: <url-redacted>, enum: RED})
+}'
+    query = GraphQL::Query.new(SanitizeTest::CustomSanitizedPrinterSchema, query_str)
+    assert_equal expected_query_string, query.sanitized_query_string
+  end
 end
 
