@@ -93,6 +93,22 @@ describe GraphQL::Query do
     end
   end
 
+  describe "when passed a query_string with an invalid type" do
+    it "returns an error to the client" do
+      assert_raises(ArgumentError) {
+        GraphQL::Query.new(schema, {"default" => "{ fromSource(source: COW) { id } }"})
+      }
+    end
+  end
+
+  describe "when passed a query with an invalid type" do
+    it "returns an error to the client" do
+      assert_raises(ArgumentError) {
+        GraphQL::Query.new(schema, query: {"default" => "{ fromSource(source: COW) { id } }"})
+      }
+    end
+  end
+
   describe "#operation_name" do
     describe "when provided" do
       let(:query_string) { <<-GRAPHQL
@@ -678,6 +694,13 @@ describe GraphQL::Query do
       query = GraphQL::Query.new(schema, invalid_query_string, validate: true)
       assert_equal false, query.valid?
       assert_equal 1, query.static_errors.length
+
+      # Can assign attribute after calling methods that use the AST
+      query = GraphQL::Query.new(schema, invalid_query_string)
+      assert query.fingerprint
+      query.validate = false
+      assert_equal true, query.valid?
+      assert_equal 0, query.static_errors.length
     end
   end
 
@@ -763,92 +786,10 @@ describe GraphQL::Query do
     end
   end
 
-  if !TESTING_INTERPRETER
-    describe '#internal_representation' do
-      let(:schema) {
-        Class.new(Dummy::Schema) do
-          query_execution_strategy(GraphQL::Execution::Execute)
-          self.interpreter = false
-          self.analysis_engine = GraphQL::Analysis
-        end
-      }
-      it "includes all definition roots" do
-        assert_kind_of GraphQL::InternalRepresentation::Node, query.internal_representation.operation_definitions["getFlavor"]
-        assert_kind_of GraphQL::InternalRepresentation::Node, query.internal_representation.fragment_definitions["cheeseFields"]
-        assert_kind_of GraphQL::InternalRepresentation::Node, query.internal_representation.fragment_definitions["edibleFields"]
-        assert_kind_of GraphQL::InternalRepresentation::Node, query.internal_representation.fragment_definitions["milkFields"]
-        assert_kind_of GraphQL::InternalRepresentation::Node, query.internal_representation.fragment_definitions["dairyFields"]
-      end
-
-      describe '#irep_selection' do
-        it "returns the irep for the selected operation" do
-          assert_kind_of GraphQL::InternalRepresentation::Node, query.irep_selection
-          assert_equal 'getFlavor', query.irep_selection.name
-        end
-
-        it "returns nil when there is no selected operation" do
-          query = GraphQL::Query.new(schema, '# Only a comment')
-          assert_nil query.irep_selection
-        end
-      end
-    end
-  end
-
-  describe "query_execution_strategy" do
-    let(:custom_execution_schema) {
-      Class.new(schema) do
-        self.interpreter = false
-        query_execution_strategy DummyStrategy
-        mutation_execution_strategy GraphQL::Execution::Execute
-        instrument(:multiplex, DummyMultiplexInstrumenter)
-      end
-    }
-
-    class DummyStrategy
-      def execute(ast_operation, root_type, query_object)
-        { "dummy" => true }
-      end
-    end
-
-    class DummyMultiplexInstrumenter
-      def self.before_multiplex(m)
-        m.queries.first.context[:before_multiplex] = true
-      end
-
-      def self.after_multiplex(m)
-      end
-    end
-
-    it "is used for running a query, if it's present and not the default" do
-      result = custom_execution_schema.execute(" { __typename }")
-      assert_equal({"data"=>{"dummy"=>true}}, result)
-
-      result = custom_execution_schema.execute(" mutation { __typename } ")
-      assert_equal({"data"=>{"__typename" => "Mutation"}}, result)
-    end
-
-    it "treats the query as a one-item multiplex" do
-      ctx = {}
-      custom_execution_schema.execute(" { __typename }", context: ctx)
-      assert_equal true, ctx[:before_multiplex]
-    end
-
-    it "can't run a multiplex" do
-      err = assert_raises ArgumentError do
-        custom_execution_schema.multiplex([
-          {query: " { __typename }"},
-          {query: " { __typename }"},
-        ])
-      end
-      msg = "Multiplexing doesn't support custom execution strategies, run one query at a time instead"
-      assert_equal msg, err.message
-    end
-  end
-
   it "Accepts a passed-in warden" do
     warden = GraphQL::Schema::Warden.new(->(t, ctx) { false }, schema: Jazz::Schema, context: nil)
     res = Jazz::Schema.execute("{ __typename } ", warden: warden)
-    assert_equal ["Field '__typename' doesn't exist on type 'Query'"], res["errors"].map { |e| e["message"] }
+    assert_equal ["Schema is not configured for queries"], res["errors"].map { |e| e["message"] }
   end
 
   describe "arguments_for" do
