@@ -9,15 +9,33 @@ describe GraphQL::Schema::Argument do
       end
     end
 
+    class ContextInput < GraphQL::Schema::InputObject
+      argument :context, String
+    end
+
+    class LoadUnauthorizedInstruments < GraphQL::Schema::Resolver
+      argument :ids, [ID], as: :instruments, required: false, loads: UnauthorizedInstrumentType
+
+      def load_instruments(ids)
+        ids.map { |id| context.schema.object_from_id(id, context).call }
+      end
+
+      type Integer, null: true
+
+      def resolve(instruments:)
+        instruments.size
+      end
+    end
+
     class Query < GraphQL::Schema::Object
-      field :field, String, null: true do
+      field :field, String do
         argument :arg, String, description: "test", required: false
         argument :deprecated_arg, String, deprecation_reason: "don't use me!", required: false
 
         argument :arg_with_block, String, required: false do
           description "test"
         end
-        argument :required_with_default_arg, Int, required: true, default_value: 1
+        argument :required_with_default_arg, Int, default_value: 1
         argument :aliased_arg, String, required: false, as: :renamed
         argument :prepared_arg, Int, required: false, prepare: :multiply
         argument :prepared_by_proc_arg, Int, required: false, prepare: ->(val, context) { context[:multiply_by] * val }
@@ -54,15 +72,21 @@ describe GraphQL::Schema::Argument do
       def multiply(val)
         context[:multiply_by] * val
       end
+
+      field :context_arg_test, [String], null: false do
+        argument :input, ContextInput
+      end
+
+      def context_arg_test(input:)
+        [input.context, input.context.class, self.context.class]
+      end
+
+      field :other_unauthorized_instruments, resolver: LoadUnauthorizedInstruments
     end
 
     class Schema < GraphQL::Schema
       query(Query)
       lazy_resolve(Proc, :call)
-      if TESTING_INTERPRETER
-        use GraphQL::Execution::Interpreter
-        use GraphQL::Analysis::AST
-      end
 
       def self.object_from_id(id, ctx)
         -> { Jazz::GloballyIdentifiableType.find(id) }
@@ -297,6 +321,16 @@ describe GraphQL::Schema::Argument do
       assert_nil res["errors"]
       assert_nil res["data"].fetch("field")
     end
+
+    it "handles applies authorization even when a custom load method is provided" do
+      query_str = <<-GRAPHQL
+      query { otherUnauthorizedInstruments(ids: ["Instrument/Drum Kit"]) }
+      GRAPHQL
+
+      res = SchemaArgumentTest::Schema.execute(query_str)
+      assert_nil res["errors"]
+      assert_nil res["data"].fetch("otherUnauthorizedInstruments")
+    end
   end
 
   describe "deprecation_reason:" do
@@ -321,7 +355,7 @@ describe GraphQL::Schema::Argument do
       err = assert_raises ArgumentError do
         Class.new(GraphQL::Schema::InputObject) do
           graphql_name 'MyInput'
-          argument :foo, String, required: true, deprecation_reason: "Don't use me"
+          argument :foo, String, deprecation_reason: "Don't use me"
         end
       end
       assert_equal "Required arguments cannot be deprecated: MyInput.foo.", err.message
@@ -347,7 +381,7 @@ describe GraphQL::Schema::Argument do
 
       query_type = Class.new(GraphQL::Schema::Object) do
         graphql_name "Query"
-        field :f, String, null: true do
+        field :f, String do
           argument :arg, input_obj, required: false
         end
       end
@@ -378,9 +412,6 @@ describe GraphQL::Schema::Argument do
           argument :lazy_object_ref, "InvalidArgumentTypeSchema::InvalidArgumentType", required: false
         end
       end
-
-      use GraphQL::Execution::Interpreter
-      use GraphQL::Analysis::AST
     end
 
     it "rejects them" do
@@ -403,4 +434,251 @@ describe GraphQL::Schema::Argument do
       assert_equal expected_message, err.message
     end
   end
+
+  describe "validating default values" do
+    it "raises when field argument default values are invalid" do
+      query_type = Class.new(GraphQL::Schema::Object) do
+        graphql_name "Query"
+        field :f1, Integer, null: false do
+          argument :arg1, Integer, default_value: nil
+        end
+      end
+
+      err = assert_raises GraphQL::Schema::Argument::InvalidDefaultValueError do
+        Class.new(GraphQL::Schema) do
+          query(query_type)
+        end
+      end
+      expected_message = "`Query.f1.arg1` has an invalid default value: `nil` isn't accepted by `Int!`; update the default value or the argument type."
+      assert_equal expected_message, err.message
+    end
+
+    it "raises when input argument default values are invalid" do
+      input_obj = Class.new(GraphQL::Schema::InputObject) do
+        graphql_name "InputObj"
+        argument :arg1, [String, null: false], default_value: [nil], required: false
+      end
+
+      query_type = Class.new(GraphQL::Schema::Object) do
+        graphql_name "Query"
+        field :f1, Integer, null: false do
+          argument :input, input_obj
+        end
+      end
+
+      err = assert_raises GraphQL::Schema::Argument::InvalidDefaultValueError do
+        Class.new(GraphQL::Schema) do
+          query(query_type)
+        end
+      end
+
+      expected_message = "`InputObj.arg1` has an invalid default value: `[nil]` isn't accepted by `[String!]`; update the default value or the argument type."
+      assert_equal expected_message, err.message
+    end
+
+    it "raises when directive argument default values are invalid" do
+      lang = Class.new(GraphQL::Schema::Enum) do
+        graphql_name "Language"
+        value "EN"
+        value "JA"
+      end
+
+      localize = Class.new(GraphQL::Schema::Directive) do
+        graphql_name "localize"
+        locations GraphQL::Schema::Directive::FIELD
+        argument :lang, lang, default_value: "ZH", required: false
+      end
+
+      err = assert_raises GraphQL::Schema::Argument::InvalidDefaultValueError do
+        Class.new(GraphQL::Schema) do
+          directive(localize)
+        end
+      end
+
+      expected_message = "`@localize.lang` has an invalid default value: `\"ZH\"` isn't accepted by `Language`; update the default value or the argument type."
+      assert_equal expected_message, err.message
+    end
+
+    it "raises when parsing a schema from a string" do
+      schema_str = <<-GRAPHQL
+      type Query {
+        f1(arg1: Int! = null): Int!
+      }
+      GRAPHQL
+
+      err = assert_raises GraphQL::Schema::Argument::InvalidDefaultValueError do
+        GraphQL::Schema.from_definition(schema_str)
+      end
+      expected_message = "`Query.f1.arg1` has an invalid default value: `nil` isn't accepted by `Int!`; update the default value or the argument type."
+      assert_equal expected_message, err.message
+
+      directive_schema_str = <<-GRAPHQL
+      enum Language {
+        EN
+        JA
+      }
+      directive @localize(lang: Language = "ZH") on FIELD
+
+      type Query {
+        f1: Int
+      }
+      GRAPHQL
+
+
+      err2 = assert_raises GraphQL::Schema::Argument::InvalidDefaultValueError do
+        GraphQL::Schema.from_definition(directive_schema_str)
+      end
+      expected_message = "`@localize.lang` has an invalid default value: `\"ZH\"` isn't accepted by `Language`; update the default value or the argument type."
+      assert_equal expected_message, err2.message
+
+      input_obj_schema_str = <<-GRAPHQL
+      input InputObj {
+        arg1: [String!] = [null]
+      }
+
+      type Query {
+        f1(arg1: InputObj): Int
+      }
+      GRAPHQL
+
+
+      err3 = assert_raises GraphQL::Schema::Argument::InvalidDefaultValueError do
+        GraphQL::Schema.from_definition(input_obj_schema_str)
+      end
+      expected_message = "`InputObj.arg1` has an invalid default value: `[nil]` isn't accepted by `[String!]`; update the default value or the argument type."
+      assert_equal expected_message, err3.message
+    end
+  end
+
+  it "works with arguments named context" do
+    res = SchemaArgumentTest::Schema.execute("{ contextArgTest(input: { context: \"abc\" }) }")
+    assert_equal ["abc", "String", "GraphQL::Query::Context"], res["data"]["contextArgTest"]
+  end
+
+  describe "required: :nullable" do
+    class RequiredNullableSchema < GraphQL::Schema
+      class Query < GraphQL::Schema::Object
+        field :echo, String do
+          argument :str, String, required: :nullable
+        end
+
+        def echo(str:)
+          str
+        end
+      end
+
+      query(Query)
+    end
+
+    it "requires a value, even if it's null" do
+      res = RequiredNullableSchema.execute('{ echo(str: "ok") }')
+      assert_equal "ok", res["data"]["echo"]
+      res = RequiredNullableSchema.execute('{ echo(str: null) }')
+      assert_nil res["data"].fetch("echo")
+      res = RequiredNullableSchema.execute('{ echo }')
+      assert_equal ["echo has the wrong arguments"], res["errors"].map { |e| e["message"] }
+    end
+  end
+
+  describe "replace_null_with_default: true" do
+    class ReplaceNullWithDefaultSchema < GraphQL::Schema
+      class Add < GraphQL::Schema::Resolver
+        argument :left, Integer, required: false, default_value: 5, replace_null_with_default: true
+        argument :right, Integer
+        type Integer, null: false
+
+        def resolve(left:, right:)
+          right + left
+        end
+      end
+
+      class AddInput < GraphQL::Schema::InputObject
+        argument :left, Integer, required: false, default_value: 5, replace_null_with_default: true
+        argument :right, Integer
+      end
+
+      class Query < GraphQL::Schema::Object
+        field :add1, Integer do
+          argument :left, Integer, required: false, default_value: 5, replace_null_with_default: true
+          argument :right, Integer
+        end
+
+        def add1(left:, right:)
+          left + right
+        end
+
+        field :add2, resolver: Add
+
+        field :add3, Integer do
+          argument :input, AddInput
+        end
+
+        def add3(input:)
+          input[:left] + input[:right]
+        end
+      end
+      query(Query)
+    end
+
+    it "works for fields, resolvers, and input objects" do
+      res1 = ReplaceNullWithDefaultSchema.execute("{ r1: add1(left: null, right: 2) r2: add1(right: 0)}")
+      assert_equal 7, res1["data"]["r1"]
+      assert_equal 5, res1["data"]["r2"]
+
+      res2 = ReplaceNullWithDefaultSchema.execute("{ r1: add2(left: null, right: 1) r2: add2(right: 3) }")
+      assert_equal 6, res2["data"]["r1"]
+      assert_equal 8, res2["data"]["r2"]
+
+      res3 = ReplaceNullWithDefaultSchema.execute("{ r1: add3(input: { left: null, right: 5 }) r2: add3(input: { right: 6 }) }")
+      assert_equal 10, res3["data"]["r1"]
+      assert_equal 11, res3["data"]["r2"]
+    end
+  end
+
+  describe "multiple argument definitions with default values" do
+    class MultipleArgumentDefaultValuesSchema < GraphQL::Schema
+      class BaseArgument < GraphQL::Schema::Argument
+        def initialize(*args, use_if:, **kwargs, &block)
+          @use_if = use_if
+          super(*args, **kwargs, &block)
+        end
+
+        def visible?(ctx)
+          ctx[:use_if] == @use_if
+        end
+      end
+
+      class BaseField < GraphQL::Schema::Field
+        argument_class BaseArgument
+      end
+
+      class Query < GraphQL::Schema::Object
+        field_class BaseField
+
+        field :echo, String do
+          argument :input, String, required: false, default_value: "argument-default-1", use_if: :visible_1
+          argument :input, String, required: false, default_value: "argument-default-2", use_if: :visible_2
+          argument :input, String, required: false, default_value: nil, use_if: :visible_3
+        end
+
+        def echo(input: "method-default")
+          input || "dynamic-fallback"
+        end
+      end
+
+      query(Query)
+    end
+
+    def get_echo_for(use_if)
+      res = MultipleArgumentDefaultValuesSchema.execute("{ echo }", context: { use_if: use_if })
+      res["data"]["echo"]
+    end
+
+    it "uses the default value from the matching argument if there is one" do
+      assert_equal "argument-default-1", get_echo_for(:visible_1)
+      assert_equal "argument-default-2", get_echo_for(:visible_2)
+      assert_equal "dynamic-fallback", get_echo_for(:visible_3)
+      assert_equal "method-default", get_echo_for(:visible_4) # no match
+    end
+  end
 end
