@@ -27,7 +27,7 @@ type HelloScalars {
 }
       SCHEMA
 
-      assert_schema_and_compare_output(schema.chop)
+      assert_schema_and_compare_output(schema)
     end
 
     it 'can build a schema with underscored names' do
@@ -41,7 +41,7 @@ type Query {
 }
       SCHEMA
 
-      assert_schema_and_compare_output(schema.chop)
+      assert_schema_and_compare_output(schema)
     end
 
     it 'can build a schema with default input object values' do
@@ -55,7 +55,7 @@ type Query {
 }
       SCHEMA
 
-      assert_schema_and_compare_output(schema.chop)
+      assert_schema_and_compare_output(schema)
     end
 
     it 'can build a schema with directives' do
@@ -66,12 +66,69 @@ schema {
 
 directive @foo(arg: Int, nullDefault: Int = null) on FIELD
 
-type Hello {
+directive @greeting(pleasant: Boolean = true) on ARGUMENT_DEFINITION | ENUM | FIELD_DEFINITION | INPUT_OBJECT | INTERFACE | OBJECT | UNION
+
+directive @greeting2 on INTERFACE
+
+directive @hashed repeatable on FIELD_DEFINITION | INPUT_FIELD_DEFINITION
+
+directive @language(is: String!) on ENUM_VALUE
+
+type Hello implements Secret @greeting {
+  goodbye(saying: Parting @greeting): Parting
+  humbug: Int @greeting(pleasant: false)
+  password: Phrase @hashed
+  str(in: Input): String
+}
+
+input Input @greeting {
+  value: String @hashed
+}
+
+enum Parting @greeting {
+  AU_REVOIR @language(is: "fr")
+  ZAI_JIAN @language(is: "zh")
+}
+
+union Phrase @greeting = Hello | Word
+
+interface Secret @greeting @greeting2 {
+  password: String
+}
+
+type Word {
   str: String
 }
       SCHEMA
 
-      assert_schema_and_compare_output(schema.chop)
+      parsed_schema = GraphQL::Schema.from_definition(schema)
+      hello_type = parsed_schema.get_type("Hello")
+      assert_equal ["deprecated", "foo", "greeting", "greeting2", "hashed", "include", "language", "skip"], parsed_schema.directives.keys.sort
+      parsed_schema.directives.values.each do |dir_class|
+        assert dir_class < GraphQL::Schema::Directive
+      end
+
+      assert_equal true, parsed_schema.directives["hashed"].repeatable?
+      assert_equal false, parsed_schema.directives["deprecated"].repeatable?
+
+      assert_equal 1, hello_type.directives.size
+      assert_instance_of parsed_schema.directives["greeting"], hello_type.directives.first
+      assert_equal({ pleasant: true }, hello_type.directives.first.arguments.keyword_arguments)
+
+      humbug_directives = hello_type.get_field("humbug").directives
+      assert_equal 1, humbug_directives.size
+      assert_instance_of parsed_schema.directives["greeting"], humbug_directives.first
+      assert_equal({ pleasant: false }, humbug_directives.first.arguments.keyword_arguments)
+
+      au_revoir_directives = parsed_schema.get_type("Parting").values["AU_REVOIR"].directives
+      assert_equal 1, au_revoir_directives.size
+      assert_instance_of parsed_schema.directives["language"], au_revoir_directives.first
+      assert_equal({ is: "fr" }, au_revoir_directives.first.arguments.keyword_arguments)
+
+      secret_type = parsed_schema.get_type("Secret")
+      assert_equal 2, secret_type.directives.size
+
+      assert_schema_and_compare_output(schema)
     end
 
     it 'supports descriptions and definition_line' do
@@ -141,7 +198,7 @@ And a union
 union U = Hello
       SCHEMA
 
-      assert_schema_and_compare_output(schema.chop)
+      assert_schema_and_compare_output(schema)
 
       built_schema = GraphQL::Schema.from_definition(schema)
       # The schema's are the same since there's no description
@@ -185,6 +242,17 @@ union U = Hello
       assert_equal 63, built_schema.types["U"].ast_node.definition_line
     end
 
+    it 'handles empty type descriptions' do
+      schema = <<-SCHEMA
+"""
+"""
+type Query {
+  f1: Int
+}
+      SCHEMA
+      refute_nil GraphQL::Schema.from_definition(schema)
+    end
+
     it 'maintains built-in directives' do
       schema = <<-SCHEMA
 schema {
@@ -259,7 +327,7 @@ type HelloScalars {
 }
       SCHEMA
 
-      assert_schema_and_compare_output(schema.chop)
+      assert_schema_and_compare_output(schema)
     end
 
     it 'supports recursive type' do
@@ -274,7 +342,7 @@ type Recurse {
 }
       SCHEMA
 
-      assert_schema_and_compare_output(schema.chop)
+      assert_schema_and_compare_output(schema)
     end
 
     it 'supports two types circular' do
@@ -294,7 +362,7 @@ type TypeTwo {
 }
       SCHEMA
 
-      assert_schema_and_compare_output(schema.chop)
+      assert_schema_and_compare_output(schema)
     end
 
     it 'supports single argument fields' do
@@ -312,7 +380,7 @@ type Hello {
 }
       SCHEMA
 
-      assert_schema_and_compare_output(schema.chop)
+      assert_schema_and_compare_output(schema)
     end
 
     it 'properly understands connections' do
@@ -426,7 +494,7 @@ type Type {
 }
       SCHEMA
 
-      built_schema = assert_schema_and_compare_output(schema.chop)
+      built_schema = assert_schema_and_compare_output(schema)
       obj = built_schema.types["Type"]
       refute obj.fields["organization"].connection?
       assert obj.fields["organizations"].connection?
@@ -443,7 +511,7 @@ type Hello {
 }
       SCHEMA
 
-      assert_schema_and_compare_output(schema.chop)
+      assert_schema_and_compare_output(schema)
     end
 
     it 'supports simple type with interface' do
@@ -461,7 +529,31 @@ interface WorldInterface {
 }
       SCHEMA
 
-      assert_schema_and_compare_output(schema.chop)
+      assert_schema_and_compare_output(schema)
+    end
+
+    it "supports interfaces that implement interfaces" do
+      schema = <<-SCHEMA
+interface Named implements Node {
+  id: ID
+  name: String
+}
+
+interface Node {
+  id: ID
+}
+
+type Query {
+  thing: Thing
+}
+
+type Thing implements Named & Node {
+  id: ID
+  name: String
+}
+      SCHEMA
+
+      assert_schema_and_compare_output(schema)
     end
 
     it 'supports simple output enum' do
@@ -479,7 +571,7 @@ type OutputEnumRoot {
 }
       SCHEMA
 
-      assert_schema_and_compare_output(schema.chop)
+      assert_schema_and_compare_output(schema)
     end
 
     it 'supports simple input enum' do
@@ -497,7 +589,7 @@ type InputEnumRoot {
 }
       SCHEMA
 
-      assert_schema_and_compare_output(schema.chop)
+      assert_schema_and_compare_output(schema)
     end
 
     it 'supports multiple value enum' do
@@ -516,7 +608,7 @@ type OutputEnumRoot {
 }
       SCHEMA
 
-      assert_schema_and_compare_output(schema.chop)
+      assert_schema_and_compare_output(schema)
     end
 
     it 'supports simple union' do
@@ -536,7 +628,7 @@ type World {
 }
       SCHEMA
 
-      assert_schema_and_compare_output(schema.chop)
+      assert_schema_and_compare_output(schema)
     end
 
     it 'supports multiple union' do
@@ -560,7 +652,7 @@ type WorldTwo {
 }
       SCHEMA
 
-      assert_schema_and_compare_output(schema.chop)
+      assert_schema_and_compare_output(schema)
     end
 
     it 'supports redefining built-in scalars' do
@@ -576,7 +668,7 @@ type Root {
 }
       SCHEMA
 
-      built_schema = assert_schema_and_compare_output(schema.chop)
+      built_schema = assert_schema_and_compare_output(schema)
       id_scalar = built_schema.types["ID"]
       assert_equal true, id_scalar.valid_isolated_input?("123")
     end
@@ -594,7 +686,7 @@ type Root {
 }
       SCHEMA
 
-      built_schema = assert_schema_and_compare_output(schema.chop)
+      built_schema = assert_schema_and_compare_output(schema)
       custom_scalar = built_schema.types["CustomScalar"]
       assert_equal true, custom_scalar.valid_isolated_input?("anything")
       assert_equal true, custom_scalar.valid_isolated_input?(12345)
@@ -616,7 +708,7 @@ type Root {
 }
       SCHEMA
 
-      assert_schema_and_compare_output(schema.chop)
+      assert_schema_and_compare_output(schema)
     end
 
     it 'supports simple argument field with default value' do
@@ -637,7 +729,7 @@ type Hello {
 }
       SCHEMA
 
-      assert_schema_and_compare_output(schema.chop)
+      assert_schema_and_compare_output(schema)
     end
 
     it 'supports simple type with mutation' do
@@ -658,7 +750,7 @@ type Mutation {
 }
       SCHEMA
 
-      assert_schema_and_compare_output(schema.chop)
+      assert_schema_and_compare_output(schema)
     end
 
     it 'supports simple type with mutation and default values' do
@@ -677,7 +769,7 @@ type Query {
 }
       SCHEMA
 
-      assert_schema_and_compare_output(schema.chop)
+      assert_schema_and_compare_output(schema)
     end
 
     it 'supports simple type with subscription' do
@@ -698,7 +790,7 @@ type Subscription {
 }
       SCHEMA
 
-      assert_schema_and_compare_output(schema.chop)
+      assert_schema_and_compare_output(schema)
     end
 
     it 'supports unreferenced type implementing referenced interface' do
@@ -716,7 +808,7 @@ type Query {
 }
       SCHEMA
 
-      assert_schema_and_compare_output(schema.chop)
+      assert_schema_and_compare_output(schema)
     end
 
     it 'supports unreferenced type implementing referenced union' do
@@ -732,11 +824,13 @@ type Query {
 union Union = Concrete
       SCHEMA
 
-      assert_schema_and_compare_output(schema.chop)
+      assert_schema_and_compare_output(schema)
     end
 
     it 'supports @deprecated' do
       schema = <<-SCHEMA
+directive @directiveWithDeprecatedArg(deprecatedArg: Boolean @deprecated(reason: "Don't use me!")) on OBJECT
+
 enum MyEnum {
   OLD_VALUE @deprecated
   OTHER_VALUE @deprecated(reason: "Terrible reasons")
@@ -756,7 +850,7 @@ type Query {
 }
       SCHEMA
 
-      assert_schema_and_compare_output(schema.chop)
+      assert_schema_and_compare_output(schema)
     end
 
     it "tracks original AST node" do
@@ -839,7 +933,7 @@ type HelloScalars {
         file.close
 
         built_schema = GraphQL::Schema.from_definition(file.path)
-        assert_equal schema.strip, GraphQL::Schema::Printer.print_schema(built_schema)
+        assert_equal schema, GraphQL::Schema::Printer.print_schema(built_schema)
       end
     end
   end
@@ -905,7 +999,7 @@ type Hello { }
 SCHEMA
 
       err = assert_raises(GraphQL::Schema::InvalidTypeError) do
-        GraphQL::Schema.from_definition(schema).to_graphql
+        GraphQL::Schema.from_definition(schema).to_graphql(silence_deprecation_warning: true)
       end
       assert_equal 'Hello is invalid: Hello must define at least 1 field. 0 defined.', err.message
     end
@@ -1345,11 +1439,28 @@ type ThingEdge {
 
       it "doesn't add arguments that aren't in the IDL" do
         schema = GraphQL::Schema.from_definition(schema_defn)
-        assert_equal schema_defn.chomp, schema.to_definition
+        assert_equal schema_defn, schema.to_definition
       end
     end
   end
 
+  it "works when a directive argument uses a redefined scalar" do
+    schema_str = <<-GRAPHQL
+      schema {
+        query: QueryRoot
+      }
+
+      directive @myDirective(id: ID) on MUTATION | QUERY
+
+      scalar ID
+
+      type QueryRoot {}
+    GRAPHQL
+
+    schema = GraphQL::Schema.from_definition(schema_str)
+    assert_equal schema.directives["myDirective"].get_argument("id").type, schema.find("ID")
+  end
+
   describe "orphan types" do
     it "only puts unreachable types in orphan types" do
       schema = GraphQL::Schema.from_definition <<-GRAPHQL
@@ -1377,7 +1488,7 @@ type ThingEdge {
 
       assert_equal [], schema.orphan_types.map(&:graphql_name)
 
-      expected_definition = <<-GRAPHQL.chomp
+      expected_definition = <<-GRAPHQL
 interface Node {
   id: ID!
 }
