@@ -13,8 +13,14 @@ describe GraphQL::Schema::Field do
       end
     end
 
+    describe "inspect" do
+      it "includes the path and return type" do
+        assert_equal "#<Jazz::BaseField Query.inspectInput(...): [String!]!>", field.inspect
+      end
+    end
+
     it "uses the argument class" do
-      arg_defn = field.graphql_definition.arguments.values.first
+      arg_defn = field.graphql_definition(silence_deprecation_warning: true).arguments.values.first
       assert_equal :ok, arg_defn.metadata[:custom]
     end
 
@@ -28,19 +34,19 @@ describe GraphQL::Schema::Field do
     end
 
     it "attaches itself to its graphql_definition as type_class" do
-      assert_equal field, field.graphql_definition.metadata[:type_class]
+      assert_equal field, field.graphql_definition(silence_deprecation_warning: true).metadata[:type_class]
     end
 
     it "camelizes the field name, unless camelize: false" do
-      assert_equal 'inspectInput', field.graphql_definition.name
+      assert_equal 'inspectInput', field.graphql_definition(silence_deprecation_warning: true).name
       assert_equal 'inspectInput', field.name
 
       underscored_field = GraphQL::Schema::Field.from_options(:underscored_field, String, null: false, camelize: false, owner: nil) do
-        argument :underscored_arg, String, required: true, camelize: false
+        argument :underscored_arg, String, camelize: false
       end
 
-      assert_equal 'underscored_field', underscored_field.to_graphql.name
-      arg_name, arg_defn = underscored_field.to_graphql.arguments.first
+      assert_equal 'underscored_field', underscored_field.deprecated_to_graphql.name
+      arg_name, arg_defn = underscored_field.deprecated_to_graphql.arguments.first
       assert_equal 'underscored_arg', arg_name
       assert_equal 'underscored_arg', arg_defn.name
     end
@@ -63,11 +69,11 @@ describe GraphQL::Schema::Field do
       object = Class.new(Jazz::BaseObject) do
         graphql_name "JustAName"
 
-        field :test, String, null: true do
-          argument :test, String, required: true
+        field :test, String do
+          argument :test, String
           description "A Description."
         end
-      end.to_graphql
+      end.deprecated_to_graphql
 
       assert_equal "test", object.fields["test"].arguments["test"].name
       assert_equal "A Description.", object.fields["test"].description
@@ -77,11 +83,11 @@ describe GraphQL::Schema::Field do
       object = Class.new(Jazz::BaseObject) do
         graphql_name "JustAName"
 
-        field :test, String, null: true do |field|
-          field.argument :test, String, required: true
+        field :test, String do |field|
+          field.argument :test, String
           field.description "A Description."
         end
-      end.to_graphql
+      end.deprecated_to_graphql
 
       assert_equal "test", object.fields["test"].arguments["test"].name
       assert_equal "A Description.", object.fields["test"].description
@@ -92,7 +98,7 @@ describe GraphQL::Schema::Field do
         graphql_name 'MyType'
       end
       field = GraphQL::Schema::Field.from_options(:my_field, type, owner: nil, null: true)
-      assert_equal type.to_graphql, field.to_graphql.type
+      assert_equal type.deprecated_to_graphql, field.deprecated_to_graphql.type
     end
 
     describe "introspection?" do
@@ -149,7 +155,7 @@ describe GraphQL::Schema::Field do
         object = Class.new(Jazz::BaseObject) do
           graphql_name "JustAName"
 
-          field :test, String, null: true, extras: [:lookahead]
+          field :test, String, extras: [:lookahead]
         end
 
         field = object.fields['test']
@@ -158,6 +164,98 @@ describe GraphQL::Schema::Field do
         assert_equal [:lookahead, :ast_node], field.extras
       end
 
+      describe "ruby argument error" do
+        class ArgumentErrorSchema < GraphQL::Schema
+          class Query < GraphQL::Schema::Object
+
+            def inspect
+              "#<#{self.class}>"
+            end
+
+            field :f1, String do
+              argument :something, Int, required: false
+            end
+
+            def f1
+              "OK"
+            end
+
+            field :f2, String, resolver_method: :field_2 do
+              argument :something, Int, required: false
+            end
+
+            def field_2(something_else: nil)
+              "ALSO OK"
+            end
+
+            field :f3, String do
+              argument :something, Int, required: false
+            end
+
+            def f3(always_missing:)
+              "NEVER OK"
+            end
+
+            field :f4, String
+
+            def f4(never_positional, ok_optional = :ok, *ok_rest)
+              "NEVER OK"
+            end
+
+            field :f5, String do
+              argument :something, Int, required: false
+            end
+
+            def f5(**ok_keyrest)
+              "OK"
+            end
+          end
+          query(Query)
+        end
+
+        it "raises a nice error when missing" do
+          assert_equal "OK", ArgumentErrorSchema.execute("{ f1 }")["data"]["f1"]
+          assert_equal "ALSO OK", ArgumentErrorSchema.execute("{ f2 }")["data"]["f2"]
+          err = assert_raises GraphQL::Schema::Field::FieldImplementationFailed do
+            ArgumentErrorSchema.execute("{ f1(something: 12) }")
+          end
+          assert_equal "Failed to call f1 on #<ArgumentErrorSchema::Query> because the Ruby method params were incompatible with the GraphQL arguments:
+
+- `something: 12` was given by GraphQL but not defined in the Ruby method. Add `something:` to the method parameters.
+", err.message
+
+          assert_instance_of ArgumentError, err.cause
+
+          err = assert_raises GraphQL::Schema::Field::FieldImplementationFailed do
+            ArgumentErrorSchema.execute("{ f2(something: 12) }")
+          end
+          assert_equal "Failed to call field_2 on #<ArgumentErrorSchema::Query> because the Ruby method params were incompatible with the GraphQL arguments:
+
+- `something: 12` was given by GraphQL but not defined in the Ruby method. Add `something:` to the method parameters.
+", err.message
+
+
+          err = assert_raises GraphQL::Schema::Field::FieldImplementationFailed do
+            ArgumentErrorSchema.execute("{ f3(something: 1) }")
+          end
+          assert_equal "Failed to call f3 on #<ArgumentErrorSchema::Query> because the Ruby method params were incompatible with the GraphQL arguments:
+
+- `something: 1` was given by GraphQL but not defined in the Ruby method. Add `something:` to the method parameters.
+- `always_missing:` is required by Ruby, but not by GraphQL. Consider `always_missing: nil` instead, or making this argument required in GraphQL.
+", err.message
+
+          err = assert_raises GraphQL::Schema::Field::FieldImplementationFailed do
+            ArgumentErrorSchema.execute("{ f4 }")
+          end
+          assert_equal "Failed to call f4 on #<ArgumentErrorSchema::Query> because the Ruby method params were incompatible with the GraphQL arguments:
+
+- `never_positional` is required by Ruby, but GraphQL doesn't pass positional arguments. If it's meant to be a GraphQL argument, use `never_positional:` instead. Otherwise, remove it.
+", err.message
+
+          assert_equal "OK", ArgumentErrorSchema.execute("{ f5(something: 2) }")["data"]["f5"]
+        end
+      end
+
       describe "argument_details" do
         class ArgumentDetailsSchema < GraphQL::Schema
           class Query < GraphQL::Schema::Object
@@ -180,8 +278,6 @@ describe GraphQL::Schema::Field do
           end
 
           query(Query)
-          use(GraphQL::Execution::Interpreter)
-          use(GraphQL::Analysis::AST)
         end
 
         it "provides metadata about arguments" do
@@ -210,7 +306,7 @@ describe GraphQL::Schema::Field do
 
     describe "type" do
       it "tells the return type" do
-        assert_equal "[String!]!", field.type.graphql_definition.to_s
+        assert_equal "[String!]!", field.type.graphql_definition(silence_deprecation_warning: true).to_s
       end
 
       it "returns the type class" do
@@ -224,8 +320,8 @@ describe GraphQL::Schema::Field do
         object = Class.new(Jazz::BaseObject) do
           graphql_name "complexityKeyword"
 
-          field :complexityTest, String, null: true, complexity: 25
-        end.to_graphql
+          field :complexityTest, String, complexity: 25
+        end.deprecated_to_graphql
 
         assert_equal 25, object.fields["complexityTest"].complexity
       end
@@ -234,10 +330,10 @@ describe GraphQL::Schema::Field do
         object = Class.new(Jazz::BaseObject) do
           graphql_name "complexityKeyword"
 
-          field :complexityTest, String, null: true do
+          field :complexityTest, String do
             complexity ->(_ctx, _args, _child_complexity) { 52 }
           end
-        end.to_graphql
+        end.deprecated_to_graphql
 
         assert_equal 52, object.fields["complexityTest"].complexity.call(nil, nil, nil)
       end
@@ -246,10 +342,10 @@ describe GraphQL::Schema::Field do
         object = Class.new(Jazz::BaseObject) do
           graphql_name "complexityKeyword"
 
-          field :complexityTest, String, null: true do
+          field :complexityTest, String do
             complexity 38
           end
-        end.to_graphql
+        end.deprecated_to_graphql
 
         assert_equal 38, object.fields["complexityTest"].complexity
       end
@@ -259,10 +355,10 @@ describe GraphQL::Schema::Field do
           Class.new(Jazz::BaseObject) do
             graphql_name "complexityKeyword"
 
-            field :complexityTest, String, null: true do
+            field :complexityTest, String do
               complexity 'One hundred and eighty'
             end
-          end.to_graphql
+          end.deprecated_to_graphql
         end
 
         assert_match /^Invalid complexity:/, err.message
@@ -273,14 +369,31 @@ describe GraphQL::Schema::Field do
           Class.new(Jazz::BaseObject) do
             graphql_name "complexityKeyword"
 
-            field :complexityTest, String, null: true do
+            field :complexityTest, String do
               complexity ->(one, two) { 52 }
             end
-          end.to_graphql
+          end.deprecated_to_graphql
         end
 
         assert_match /^A complexity proc should always accept 3 parameters/, err.message
       end
+
+      it 'fails if second argument is a mutation instead of a type' do
+        mutation_class = Class.new(GraphQL::Schema::Mutation) do
+          graphql_name "Thing"
+          field :stuff, String, null: false
+        end
+
+        err = assert_raises(ArgumentError) do
+          Class.new(Jazz::BaseObject) do
+            graphql_name "complexityKeyword"
+
+            field :complexityTest, mutation_class
+          end
+        end
+
+        assert_match /^Use `field :complexityTest, mutation: Mutation, ...` to provide a mutation to this field instead/, err.message
+      end
     end
   end
 
@@ -292,7 +405,7 @@ describe GraphQL::Schema::Field do
         field :stuff, Set, null: false
       end
 
-      err = assert_raises ArgumentError do
+      err = assert_raises GraphQL::Schema::Field::MissingReturnTypeError do
         thing.fields["stuff"].type
       end
 
@@ -301,7 +414,7 @@ describe GraphQL::Schema::Field do
     end
 
     it "makes a suggestion when the type is false" do
-      err = assert_raises ArgumentError do
+      err = assert_raises GraphQL::Schema::Field::MissingReturnTypeError do
         Class.new(GraphQL::Schema::Object) do
           graphql_name "Thing"
           # False might come from an invalid `!`
@@ -312,18 +425,6 @@ describe GraphQL::Schema::Field do
       assert_includes err.message, "Thing.stuff"
       assert_includes err.message, "Received `false` instead of a type, maybe a `!` should be replaced with `null: true` (for fields) or `required: true` (for arguments)"
     end
-
-    it "makes a suggestion when the type is a GraphQL::Field" do
-      err = assert_raises ArgumentError do
-        Class.new(GraphQL::Schema::Object) do
-          graphql_name "Thing"
-          # Previously, field was a valid second argument
-          field :stuff, GraphQL::Relay::Node.field, null: false
-        end
-      end
-
-      assert_includes err.message, "use the `field:` keyword for this instead"
-    end
   end
 
   describe "mutation" do
@@ -381,8 +482,8 @@ describe GraphQL::Schema::Field do
       class Query < GraphQL::Schema::Object
         field_class BaseField
 
-        field :company, Company, null: true do
-          argument :id, ID, required: true
+        field :company, Company do
+          argument :id, ID
         end
 
         def company(id:)
@@ -429,6 +530,76 @@ describe GraphQL::Schema::Field do
     end
   end
 
+  describe "retrieving nested hash keys using dig" do
+    class DigSchema < GraphQL::Schema
+      class PersonType < GraphQL::Schema::Object
+        field :name, String, null: false
+      end
+
+      class MovieType < GraphQL::Schema::Object
+        field :title, String, null: false, dig: [:title]
+        field :stars, [PersonType], null: false, dig: ["credits", "stars"]
+        field :metascore, Float, null: false, dig: [:meta, "metascore"]
+        field :release_date, GraphQL::Types::ISO8601DateTime, null: false, dig: [:meta, :release_date]
+        field :includes_wilhelm_scream, Boolean, null: false, dig: [:meta, "wilhelm_scream"]
+        field :nullable_field, String, dig: [:this_should, :work_since, :dig_handles, :safe_expansion]
+      end
+
+      class QueryType < GraphQL::Schema::Object
+        field :a_good_laugh, MovieType, null: false
+        def a_good_laugh
+          {
+            :title => "Monty Python and the Holy Grail",
+            :meta => {
+              "metascore" => 91,
+              :release_date => DateTime.new(1975, 5, 25, 0, 0, 0),
+              "wilhelm_scream" => false
+            },
+            "credits" => {
+              "stars" => [
+                { :name => "Graham Chapman" },
+                { :name => "John Cleese" }
+              ]
+            }
+          }
+        end
+      end
+
+      query(QueryType)
+    end
+
+    it "finds the expected data" do
+      res = DigSchema.execute <<-GRAPHQL
+      {
+        aGoodLaugh {
+          title
+          includesWilhelmScream
+          metascore
+          nullableField
+          releaseDate
+          stars {
+            name
+          }
+        }
+      }
+      GRAPHQL
+
+      result = res["data"]["aGoodLaugh"]
+      expected_result = {
+        "title" => "Monty Python and the Holy Grail",
+        "includesWilhelmScream" => false,
+        "metascore" => 91.0,
+        "nullableField" => nil,
+        "releaseDate" => "1975-05-25T00:00:00+00:00",
+        "stars" => [
+          { "name" => "Graham Chapman" },
+          { "name" => "John Cleese" }
+        ]
+      }
+      assert_equal expected_result, result
+    end
+  end
+
   describe "looking up hash keys with case" do
     class HashKeySchema < GraphQL::Schema
       class ResultType < GraphQL::Schema::Object
@@ -475,4 +646,19 @@ describe GraphQL::Schema::Field do
       assert_equal expected_result, search_results
     end
   end
+
+  describe "when the owner is nil" do
+    it "raises a descriptive error" do
+      bad_field = GraphQL::Schema::Field.new(name: "something", owner: nil, type: String)
+      assert_nil bad_field.owner
+      err = assert_raises GraphQL::InvariantError do
+        bad_field.owner_type
+      end
+      expected_message = "Field \"something\" (graphql name: \"something\") has no owner, but all fields should have an owner. How did this happen?!
+
+This is probably a bug in GraphQL-Ruby, please report this error on GitHub: https://github.com/rmosolgo/graphql-ruby/issues/new?template=bug_report.md"
+
+      assert_equal expected_message, err.message
+    end
+  end
 end
