@@ -21,6 +21,14 @@ describe GraphQL::Schema::InputObject do
       assert_equal 1, input_object.arguments.size
     end
 
+    it "returns newly-added argument definitions" do
+      arg = nil
+      Class.new(GraphQL::Schema::InputObject) do
+        arg = argument(:int, Integer)
+      end
+      assert_instance_of GraphQL::Schema::Argument, arg
+    end
+
     it "is the #owner of its arguments" do
       argument = input_object.arguments["name"]
       assert_equal input_object, argument.owner
@@ -28,13 +36,13 @@ describe GraphQL::Schema::InputObject do
 
     it "inherits arguments" do
       base_class = Class.new(GraphQL::Schema::InputObject) do
-        argument :arg1, String, required: true
-        argument :arg2, String, required: true
+        argument :arg1, String
+        argument :arg2, String
       end
 
       subclass = Class.new(base_class) do
-        argument :arg2, Integer, required: true
-        argument :arg3, Integer, required: true
+        argument :arg2, Integer
+        argument :arg3, Integer
       end
 
       ensemble_class = Class.new(subclass) do
@@ -51,12 +59,12 @@ describe GraphQL::Schema::InputObject do
 
   describe ".to_graphql" do
     it "assigns itself as the arguments_class" do
-      assert_equal input_object, input_object.to_graphql.arguments_class
+      assert_equal input_object, input_object.deprecated_to_graphql.arguments_class
     end
 
     it "accepts description: kwarg" do
       input_obj_class = Jazz::InspectableInput
-      input_obj_type = input_obj_class.to_graphql
+      input_obj_type = input_obj_class.deprecated_to_graphql
       assert_equal "Test description kwarg", input_obj_type.arguments["stringValue"].description
     end
   end
@@ -89,7 +97,7 @@ describe GraphQL::Schema::InputObject do
   describe "prepare with camelized inputs" do
     class PrepareCamelizedSchema < GraphQL::Schema
       class CamelizedInput < GraphQL::Schema::InputObject
-        argument :inputString, String, required: true,
+        argument :inputString, String,
           camelize: false,
           as: :input_string,
           prepare: ->(val, ctx) { val.upcase }
@@ -97,7 +105,7 @@ describe GraphQL::Schema::InputObject do
 
       class Query < GraphQL::Schema::Object
         field :input_test, String, null: false do
-          argument :camelized_input, CamelizedInput, required: true
+          argument :camelized_input, CamelizedInput
         end
 
         def input_test(camelized_input:)
@@ -106,10 +114,6 @@ describe GraphQL::Schema::InputObject do
       end
 
       query(Query)
-      if TESTING_INTERPRETER
-        use GraphQL::Execution::Interpreter
-        use GraphQL::Analysis::AST
-      end
     end
 
     it "calls the prepare proc" do
@@ -121,12 +125,12 @@ describe GraphQL::Schema::InputObject do
   describe "prepare: / loads: / as:" do
     module InputObjectPrepareTest
       class InputObj < GraphQL::Schema::InputObject
-        argument :a, Integer, required: true
-        argument :b, Integer, required: true, as: :b2
-        argument :c, Integer, required: true, prepare: :prep
-        argument :d, Integer, required: true, prepare: :prep, as: :d2
-        argument :e, Integer, required: true, prepare: ->(val, ctx) { val * ctx[:multiply_by] * 2 }, as: :e2
-        argument :instrument_id, ID, required: true, loads: Jazz::InstrumentType
+        argument :a, Integer
+        argument :b, Integer, as: :b2
+        argument :c, Integer, prepare: :prep
+        argument :d, Integer, prepare: :prep, as: :d2
+        argument :e, Integer, prepare: ->(val, ctx) { val * ctx[:multiply_by] * 2 }, as: :e2
+        argument :instrument_id, ID, loads: Jazz::InstrumentType
         argument :danger, Integer, required: false, prepare: ->(val, ctx) { raise GraphQL::ExecutionError.new('boom!') }
 
         def prep(val)
@@ -136,7 +140,7 @@ describe GraphQL::Schema::InputObject do
 
       class Query < GraphQL::Schema::Object
         field :inputs, [String], null: false do
-          argument :input, InputObj, required: true
+          argument :input, InputObj
         end
 
         def inputs(input:)
@@ -146,11 +150,11 @@ describe GraphQL::Schema::InputObject do
 
       class Mutation < GraphQL::Schema::Object
         class InstrumentInput < GraphQL::Schema::InputObject
-          argument :instrument_id, ID, required: true, loads: Jazz::InstrumentType
+          argument :instrument_id, ID, loads: Jazz::InstrumentType
         end
 
         class TouchInstrument < GraphQL::Schema::Mutation
-          argument :input_obj, InstrumentInput, required: true
+          argument :input_obj, InstrumentInput
           field :instrument_name_method, String, null: false
           field :instrument_name_key, String, null: false
 
@@ -163,10 +167,26 @@ describe GraphQL::Schema::InputObject do
           end
         end
 
+        class InstrumentByNameInput < GraphQL::Schema::InputObject
+          argument :instrument_name, ID, loads: Jazz::InstrumentType, as: :instrument
+
+          def self.load_instrument(instrument_name, context)
+            -> {
+              instruments = Jazz::Models.data["Instrument"]
+              instruments.find { |i| i.name == instrument_name }
+            }
+          end
+        end
+
+        class TouchInstrumentByName < TouchInstrument
+          argument :input_obj, InstrumentByNameInput
+        end
+
         field :touch_instrument, mutation: TouchInstrument
+        field :touch_instrument_by_name, mutation: TouchInstrumentByName
 
         class ListInstruments < GraphQL::Schema::Mutation
-          argument :list, [InstrumentInput], required: true
+          argument :list, [InstrumentInput]
           field :resolved_list, String, null: false
 
           def resolve(list:)
@@ -179,15 +199,10 @@ describe GraphQL::Schema::InputObject do
         field :list_instruments, mutation: ListInstruments
       end
 
-
       class Schema < GraphQL::Schema
         query(Query)
         mutation(Mutation)
         lazy_resolve(Proc, :call)
-        if TESTING_INTERPRETER
-          use GraphQL::Execution::Interpreter
-          use GraphQL::Analysis::AST
-        end
 
         def self.object_from_id(id, ctx)
           -> { Jazz::GloballyIdentifiableType.find(id) }
@@ -212,6 +227,21 @@ describe GraphQL::Schema::InputObject do
       assert_equal expected_obj, res["data"]["inputs"]
     end
 
+    it "calls load_ methods for arguments when they're present" do
+      query_str = <<-GRAPHQL
+      mutation {
+        touchInstrumentByName(inputObj: { instrumentName: "Flute" }) {
+          instrumentNameMethod
+          instrumentNameKey
+        }
+      }
+      GRAPHQL
+
+      res = InputObjectPrepareTest::Schema.execute(query_str)
+      assert_equal "Flute", res["data"]["touchInstrumentByName"]["instrumentNameMethod"]
+      assert_equal "Flute", res["data"]["touchInstrumentByName"]["instrumentNameKey"]
+    end
+
     it "handles exceptions preparing variable input objects" do
       query_str = <<-GRAPHQL
       query($input: InputObj!){ inputs(input: $input) }
@@ -222,15 +252,8 @@ describe GraphQL::Schema::InputObject do
                                                    variables: { input: input})
       assert_nil(res["data"])
 
-      if TESTING_INTERPRETER
-        assert_equal("boom!", res["errors"][0]["message"])
-        assert_equal([{ "line" => 1, "column" => 33 }], res["errors"][0]["locations"])
-      else
-        assert_equal("Variable $input of type InputObj! was provided invalid value", res["errors"][0]["message"])
-        assert_equal([{ "line" => 1, "column" => 13 }], res["errors"][0]["locations"])
-        assert_equal("boom!", res["errors"][0]["extensions"]["problems"][0]["explanation"])
-        assert_equal(input, res["errors"][0]["extensions"]["value"])
-      end
+      assert_equal("boom!", res["errors"][0]["message"])
+      assert_equal([{ "line" => 1, "column" => 33 }], res["errors"][0]["locations"])
     end
 
     it "handles not-found with max complexity analyzer running" do
@@ -278,32 +301,63 @@ describe GraphQL::Schema::InputObject do
 
   describe "prepare (entire input object)" do
     module InputObjectPrepareObjectTest
-      class InputObj < GraphQL::Schema::InputObject
-        argument :min, Integer, required: true
+      class RangeInput < GraphQL::Schema::InputObject
+        argument :min, Integer
         argument :max, Integer, required: false
 
+        def self.authorized?(obj, arg, ctx)
+          if arg.end <= 100
+            super
+          else
+            ctx.add_error(GraphQL::ExecutionError.new("Range too big"))
+            false
+          end
+        end
+
         def prepare
           min..max
         end
       end
 
+      class SmallIntegerArgument < GraphQL::Schema::Argument
+        def authorized?(obj, val, ctx)
+          if val > 100
+            raise GraphQL::ExecutionError, "#{path} too big"
+          else
+            super
+          end
+        end
+      end
+
+      class HashInput < GraphQL::Schema::InputObject
+        argument_class SmallIntegerArgument
+        argument :k1, Integer
+        argument :k2, Integer
+      end
+
+      class HashInputResolver < GraphQL::Schema::Resolver
+        argument :input, HashInput, prepare: ->(val, ctx) { val.to_h }
+        type String, null: true
+
+        def resolve(input:)
+          "#{input.class}, #{input.inspect}"
+        end
+      end
+
       class Query < GraphQL::Schema::Object
-        field :inputs, String, null: false do
-          argument :input, InputObj, required: true
+        field :inputs, String do
+          argument :input, RangeInput
         end
 
         def inputs(input:)
           input.inspect
         end
+
+        field :hash_input, resolver: HashInputResolver
       end
 
       class Schema < GraphQL::Schema
         query(Query)
-
-        if TESTING_INTERPRETER
-          use GraphQL::Execution::Interpreter
-          use GraphQL::Analysis::AST
-        end
       end
     end
 
@@ -313,18 +367,37 @@ describe GraphQL::Schema::InputObject do
       GRAPHQL
 
       res = InputObjectPrepareObjectTest::Schema.execute(query_str)
-      expected_obj = (5..10).inspect
-      assert_equal expected_obj, res["data"]["inputs"]
+      assert_equal "5..10", res["data"]["inputs"]
     end
 
     it "calls prepare on the input object (variable)" do
       query_str = <<-GRAPHQL
-      query ($input: InputObj!){ inputs(input: $input) }
+      query ($input: RangeInput!){ inputs(input: $input) }
       GRAPHQL
 
       res = InputObjectPrepareObjectTest::Schema.execute(query_str, variables: { input: { min: 5, max: 10 } })
-      expected_obj = (5..10).inspect
-      assert_equal expected_obj, res["data"]["inputs"]
+      assert_equal "5..10", res["data"]["inputs"]
+    end
+
+    it "authorizes the prepared value" do
+      query_str = <<-GRAPHQL
+        query ($input: RangeInput!){ inputs(input: $input) }
+      GRAPHQL
+
+      res = InputObjectPrepareObjectTest::Schema.execute(query_str, variables: { input: { min: 5, max: 101 } })
+      assert_nil res["data"]["inputs"]
+      assert_equal ["Range too big"], res["errors"].map { |e| e["message"] }
+    end
+
+    it "authorizes Hashes returned from prepare:" do
+      query_str = "{ hashInput(input: { k1: 5, k2: 12 }) }"
+      res = InputObjectPrepareObjectTest::Schema.execute(query_str)
+      assert_equal "Hash, {:k1=>5, :k2=>12}", res["data"]["hashInput"]
+
+      query_str = "{ hashInput(input: { k1: 500, k2: 12 }) }"
+      res = InputObjectPrepareObjectTest::Schema.execute(query_str)
+      assert_equal ["HashInput.k1 too big"], res["errors"].map { |e| e["message"] }
+      assert_nil res["data"]["hashInput"]
     end
   end
 
@@ -350,20 +423,20 @@ describe GraphQL::Schema::InputObject do
 
       class SingleLoadInputObj < GraphQL::Schema::InputObject
         argument_class BaseArgument
-        argument :instrument_id, ID, required: true, loads: Jazz::InstrumentType
+        argument :instrument_id, ID, loads: Jazz::InstrumentType
       end
 
       class MultiLoadInputObj < GraphQL::Schema::InputObject
         argument_class BaseArgument
-        argument :instrument_ids, [ID], required: true, loads: Jazz::InstrumentType
+        argument :instrument_ids, [ID], loads: Jazz::InstrumentType
       end
 
       class Query < GraphQL::Schema::Object
-        field :single_load_input, Jazz::InstrumentType, null: true do
-          argument :input, SingleLoadInputObj, required: true
+        field :single_load_input, Jazz::InstrumentType do
+          argument :input, SingleLoadInputObj
         end
-        field :multi_load_input, [Jazz::InstrumentType], null: true do
-          argument :input, MultiLoadInputObj, required: true
+        field :multi_load_input, [Jazz::InstrumentType] do
+          argument :input, MultiLoadInputObj
         end
 
         def single_load_input(input:)
@@ -385,11 +458,6 @@ describe GraphQL::Schema::InputObject do
         def self.resolve_type(type, obj, ctx)
           type
         end
-
-        if TESTING_INTERPRETER
-          use GraphQL::Analysis::AST
-          use GraphQL::Execution::Interpreter
-        end
       end
     end
 
@@ -512,30 +580,28 @@ describe GraphQL::Schema::InputObject do
 
     it "works with empty objects" do
       res = Jazz::Schema.execute("{ defaultValueTest2 }")
-      assert_equal "Jazz::InspectableInput -> {}", res["data"]["defaultValueTest2"]
+      assert_equal "Jazz::FullyOptionalInput -> {}", res["data"]["defaultValueTest2"]
     end
 
     it "introspects in GraphQL language with enums" do
       class InputDefaultSchema < GraphQL::Schema
         class Letter < GraphQL::Schema::Enum
-          value "A"
-          value "B"
+          value "A", value: 1
+          value "B", value: 2
         end
 
         class InputObj < GraphQL::Schema::InputObject
-          argument :a, Letter, required: false
-          argument :b, Letter, required: false
+          argument :arg_a, Letter, required: false
+          argument :arg_b, Letter, required: false
         end
 
         class Query < GraphQL::Schema::Object
-          field :i, Int, null: true do
-            argument :arg, InputObj, required: false, default_value: { a: "A", b: "B" }
+          field :i, Int do
+            argument :arg, InputObj, required: false, default_value: { arg_a: 1, arg_b: 2 }
           end
         end
 
         query(Query)
-        use GraphQL::Execution::Interpreter
-        use GraphQL::Analysis::AST
       end
 
       res = InputDefaultSchema.execute "
@@ -551,7 +617,7 @@ describe GraphQL::Schema::InputObject do
         }
       }
       "
-      assert_equal "{a: A, b: B}", res["data"]["__type"]["fields"].first["args"].first["defaultValue"]
+      assert_equal "{argA: A, argB: B}", res["data"]["__type"]["fields"].first["args"].first["defaultValue"]
     end
   end
 
@@ -559,28 +625,38 @@ describe GraphQL::Schema::InputObject do
     module InputObjectToHTest
       class TestInput1 < GraphQL::Schema::InputObject
         graphql_name "TestInput1"
-        argument :d, Int, required: true
-        argument :e, Int, required: true
-        argument :instrument_id, ID, required: true, loads: Jazz::InstrumentType
+        argument :d, Int
+        argument :e, Int
+        argument :instrument_id, ID, loads: Jazz::InstrumentType
       end
 
       class TestInput2 < GraphQL::Schema::InputObject
         graphql_name "TestInput2"
-        argument :a, Int, required: true
-        argument :b, Int, required: true
-        argument :c, TestInput1, as: :inputObject, required: true
+        argument :a, Int
+        argument :b, Int
+        argument :c, TestInput1, as: :inputObject
       end
 
-      TestInput1.to_graphql
-      TestInput2.to_graphql
+      TestInput1.deprecated_to_graphql
+      TestInput2.deprecated_to_graphql
     end
 
     before do
       arg_values = {a: 1, b: 2, c: { d: 3, e: 4, instrumentId: "Instrument/Drum Kit"}}
 
+      query = GraphQL::Query.new(Jazz::Schema, "{ __typename }")
+      # This is because a test below expects `instrument:` to have been loaded
+      # during `InputObject#initialize`, which only happens when `!context.interpreter?`.
+      # Maybe that test could be updated instead.
+      context_without_interpreter = Class.new(SimpleDelegator) do
+        def interpreter?
+          false
+        end
+      end
+
       @input_object = InputObjectToHTest::TestInput2.new(
         arg_values,
-        context: OpenStruct.new(warden: Jazz::Schema, schema: Jazz::Schema),
+        context: context_without_interpreter.new(query.context),
         defaults_used: Set.new
       )
     end
@@ -602,19 +678,19 @@ describe GraphQL::Schema::InputObject do
     module InputObjectDigTest
       class TestInput1 < GraphQL::Schema::InputObject
         graphql_name "TestInput1"
-        argument :d, Int, required: true
-        argument :e, Int, required: true
+        argument :d, Int
+        argument :e, Int
       end
 
       class TestInput2 < GraphQL::Schema::InputObject
         graphql_name "TestInput2"
-        argument :a, Int, required: true
-        argument :b, Int, required: true
-        argument :c, TestInput1, as: :inputObject, required: true
+        argument :a, Int
+        argument :b, Int
+        argument :c, TestInput1, as: :inputObject
       end
 
-      TestInput1.to_graphql
-      TestInput2.to_graphql
+      TestInput1.deprecated_to_graphql
+      TestInput2.deprecated_to_graphql
     end
     arg_values = {a: 1, b: 2, c: { d: 3, e: 4 }}
 
@@ -676,23 +752,26 @@ describe GraphQL::Schema::InputObject do
     it "warns for method conflicts" do
       input_object = Class.new(GraphQL::Schema::InputObject) do
         graphql_name "X"
-        argument :method, String, required: true
+        argument :method, String
       end
 
-      expected_warning = "Unable to define a helper for argument with name 'method' as this is a reserved name. Add `method_access: false` to stop this warning.\n"
-      assert_output "", expected_warning do
-        input_object.graphql_definition
+      expected_warning = "Unable to define a helper for argument with name 'method' as this is a reserved name. Add `method_access: false` to stop this warning."
+
+      messages = []
+      GraphQL::Deprecation.stub(:warn, ->(message) { messages << message; nil }) do
+        input_object.graphql_definition(silence_deprecation_warning: true)
       end
+      assert_equal [expected_warning], messages
     end
 
     it "doesn't warn with `method_access: false`" do
       input_object = Class.new(GraphQL::Schema::InputObject) do
         graphql_name "X"
-        argument :method, String, required: true, method_access: false
+        argument :method, String, method_access: false
       end
 
       assert_output "", "" do
-        input_object.graphql_definition
+        input_object.graphql_definition(silence_deprecation_warning: true)
       end
     end
   end
@@ -700,11 +779,11 @@ describe GraphQL::Schema::InputObject do
   describe "nested objects inside lists" do
     class NestedInputObjectSchema < GraphQL::Schema
       class ItemInput < GraphQL::Schema::InputObject
-        argument :str, String, required: true
+        argument :str, String
       end
 
       class NestedStuff < GraphQL::Schema::RelayClassicMutation
-        argument :items, [ItemInput], required: true
+        argument :items, [ItemInput]
         field :str, String, null: false
         def resolve(items:)
           {
@@ -718,8 +797,6 @@ describe GraphQL::Schema::InputObject do
       end
 
       mutation(Mutation)
-      use GraphQL::Analysis::AST
-      use GraphQL::Execution::Interpreter
     end
 
     it "properly wraps them in instances" do
@@ -733,4 +810,392 @@ describe GraphQL::Schema::InputObject do
       assert_equal "NestedInputObjectSchema::ItemInput, NestedInputObjectSchema::ItemInput", res["data"]["nestedStuff"]["str"]
     end
   end
+
+  describe 'default values' do
+    describe 'when the type is an enum with underlying ruby values' do
+      it 'provides the default value' do
+        class TestEnum < GraphQL::Schema::Enum
+          value 'A', 'Represents an authorized agent in our system.', value: 'a'
+          value 'B', 'Agent is disabled, web app access is denied.', value: 'b'
+        end
+
+        class TestInput < GraphQL::Schema::InputObject
+          argument :foo, TestEnum, 'TestEnum', required: false, default_value: 'a'
+        end
+
+        test_input_type = TestInput
+        default_test_input_value = test_input_type.coerce_isolated_input({})
+        assert_equal default_test_input_value[:foo], 'a'
+      end
+    end
+
+    describe "when it's an empty object" do
+      it "is passed in" do
+        input_obj = Class.new(GraphQL::Schema::InputObject) do
+          graphql_name "InputObj"
+          argument :s, String, required: false
+        end
+
+        query = Class.new(GraphQL::Schema::Object) do
+          graphql_name "Query"
+          field :f, String do
+            argument :arg, input_obj, required: false, default_value: {}
+          end
+
+          def f(arg:)
+            arg.to_h.inspect
+          end
+        end
+
+        schema = Class.new(GraphQL::Schema) do
+          query(query)
+        end
+
+        res = schema.execute("{ f } ")
+        assert_equal "{}", res["data"]["f"]
+      end
+    end
+  end
+
+  describe "tests from legacy input_object_type" do
+    let(:input_object) { Dummy::DairyProductInput }
+
+    def validate_isolated_input(t, input)
+      t.validate_input(input, GraphQL::Query::NullContext)
+    end
+
+    describe "input validation" do
+      it "Accepts anything that yields key-value pairs to #all?" do
+        values_obj = MinimumInputObject.new({"source" => "COW", "fatContent" => 0.4})
+        assert input_object.valid_isolated_input?(values_obj)
+      end
+
+      describe "validate_input with non-enumerable input" do
+        it "returns a valid result for MinimumInputObject" do
+          result = validate_isolated_input(input_object, MinimumInputObject.new({"source" => "COW", "fatContent" => 0.4}))
+          assert(result.valid?)
+        end
+
+        it "returns an invalid result for MinimumInvalidInputObject" do
+          invalid_input = MinimumInputObject.new({"source" => "KOALA", "fatContent" => 0.4})
+          result = validate_isolated_input(input_object, invalid_input)
+          assert(!result.valid?)
+        end
+      end
+
+      describe "validate_input with null" do
+        let(:schema) { GraphQL::Schema.from_definition(%|
+          type Query {
+            a: Int
+          }
+
+          input ExampleInputObject {
+            a: String
+            b: Int!
+          }
+        |) }
+        let(:input_type) { schema.types['ExampleInputObject'] }
+
+        it "returns an invalid result when value is null for non-null argument" do
+          invalid_input = MinimumInputObject.new({"a" => "Test", "b" => nil})
+          result = validate_isolated_input(input_type, invalid_input)
+          assert(!result.valid?)
+        end
+
+        it "returns valid result when value is null for nullable argument" do
+          invalid_input = MinimumInputObject.new({"a" => nil, "b" => 1})
+          result = validate_isolated_input(input_type, invalid_input)
+          assert(result.valid?)
+        end
+      end
+
+      describe "validate_input with enumerable input" do
+        describe "with good input" do
+          it "returns a valid result" do
+            input = {
+              "source" => "COW",
+              "fatContent" => 0.4
+            }
+            result = validate_isolated_input(input_object, input)
+            assert(result.valid?)
+          end
+        end
+
+        if testing_rails? && ActionPack::VERSION::MAJOR > 3
+          it "returns a valid result" do
+            input = ActionController::Parameters.new(
+              "source" => "COW",
+              "fatContent" => 0.4,
+            )
+            result = validate_isolated_input(input_object, input)
+            assert_nil result.problems
+            assert(result.valid?)
+          end
+        end
+
+        describe "with bad enum and float" do
+          let(:result) { validate_isolated_input(input_object, {"source" => "KOALA", "fatContent" => "bad_num"}) }
+
+          it "returns an invalid result" do
+            assert(!result.valid?)
+          end
+
+          it "has problems with correct paths" do
+            paths = result.problems.map { |p| p["path"] }
+            assert(paths.include?(["source"]))
+            assert(paths.include?(["fatContent"]))
+          end
+
+          it "has correct problem explanation" do
+            expected = validate_isolated_input(Dummy::DairyAnimal, "KOALA").problems[0]["explanation"]
+
+            source_problem = result.problems.detect { |p| p["path"] == ["source"] }
+            actual = source_problem["explanation"]
+
+            assert_equal(expected, actual)
+          end
+        end
+
+        describe 'with a string as input' do
+          let(:result) { validate_isolated_input(input_object, "just a string") }
+
+          it "returns an invalid result" do
+            assert(!result.valid?)
+          end
+
+          it "has problem with correct path" do
+            paths = result.problems.map { |p| p["path"] }
+            assert(paths.include?([]))
+          end
+
+          it "has correct problem explanation" do
+            assert_includes result.problems[0]["explanation"], "to be a key-value object"
+          end
+        end
+
+        describe 'with an array as input' do
+          let(:result) { validate_isolated_input(input_object, ["string array"]) }
+
+          it "returns an invalid result" do
+            assert(!result.valid?)
+          end
+
+          it "has problem with correct path" do
+            paths = result.problems.map { |p| p["path"] }
+            assert(paths.include?([]))
+          end
+
+          it "has correct problem explanation" do
+            assert_includes result.problems[0]["explanation"], "to be a key-value object"
+          end
+        end
+
+        describe 'with a int as input' do
+          let(:result) { validate_isolated_input(input_object, 10) }
+
+          it "returns an invalid result" do
+            assert(!result.valid?)
+          end
+
+          it "has problem with correct path" do
+            paths = result.problems.map { |p| p["path"] }
+            assert(paths.include?([]))
+          end
+
+          it "has correct problem explanation" do
+            assert_includes result.problems[0]["explanation"], "to be a key-value object"
+          end
+        end
+
+        describe "with extra argument" do
+          let(:result) { validate_isolated_input(input_object, {"source" => "COW", "fatContent" => 0.4, "isDelicious" => false}) }
+
+          it "returns an invalid result" do
+            assert(!result.valid?)
+          end
+
+          it "has problem with correct path" do
+            paths = result.problems.map { |p| p["path"] }
+            assert_equal([["isDelicious"]], paths)
+          end
+
+          it "has correct problem explanation" do
+            assert(result.problems[0]["explanation"].include?("Field is not defined"))
+          end
+        end
+
+        describe "list with one invalid element" do
+          let(:list_type) { Dummy::DairyProductInput.to_list_type }
+          let(:result) do
+            validate_isolated_input(list_type, [
+              { "source" => "COW", "fatContent" => 0.4 },
+              { "source" => "KOALA", "fatContent" => 0.4 }
+            ])
+          end
+
+          it "returns an invalid result" do
+            assert(!result.valid?)
+          end
+
+          it "has one problem" do
+            assert_equal(result.problems.length, 1)
+          end
+
+          it "has problem with correct path" do
+            path = result.problems[0]["path"]
+            assert_equal(path, [1, "source"])
+          end
+
+          it "has problem with correct explanation" do
+            expected = validate_isolated_input(Dummy::DairyAnimal, "KOALA").problems[0]["explanation"]
+            actual = result.problems[0]["explanation"]
+            assert_equal(expected, actual)
+          end
+        end
+      end
+    end
+
+    describe "coerce_result" do
+      it "omits unspecified arguments" do
+        result = input_object.coerce_isolated_result({fatContent: 0.3})
+        assert_equal ["fatContent"], result.keys
+        assert_equal 0.3, result["fatContent"]
+      end
+    end
+
+    describe "coercion of null inputs" do
+      let(:schema) { GraphQL::Schema.from_definition(%|
+        type Query {
+          a: Int
+        }
+
+        input ExampleInputObject {
+          a: String
+          b: Int!
+          c: String = "Default"
+          d: Boolean = false
+        }
+
+        input SecondLevelInputObject {
+          example: ExampleInputObject = {b: 42, d: true}
+        }
+      |) }
+      let(:input_type) { schema.types['ExampleInputObject'] }
+
+      it "null values are returned in coerced input" do
+        input = MinimumInputObject.new({"a" => "Test", "b" => nil,"c" => "Test"})
+        err = assert_raises GraphQL::ExecutionError do
+          input_type.coerce_isolated_input(input)
+        end
+
+        assert_equal "`null` is not a valid input for `Int!`, please provide a value for this argument.", err.message
+      end
+
+      it "null values are preserved when argument has a default value" do
+        input = MinimumInputObject.new({"a" => "Test", "b" => 1, "c" => nil})
+        result = input_type.coerce_isolated_input(input)
+
+        assert_equal 'Test', result[:a]
+        assert_equal 1, result[:b]
+
+        assert result.key?(:c)
+        assert_nil result[:c]
+      end
+
+      it "omitted arguments are not returned" do
+        input = MinimumInputObject.new({"b" => 1, "c" => "Test"})
+        result = input_type.coerce_isolated_input(input)
+
+        assert !result.key?(:a)
+        assert_equal 1, result[:b]
+        assert_equal 'Test', result[:c]
+      end
+
+      it "false default values are returned" do
+        input = MinimumInputObject.new({"b" => 1})
+        result = input_type.coerce_isolated_input(input)
+
+        assert_equal false, result[:d]
+      end
+
+      it "merges defaults of nested input objects" do
+        result = schema.types['SecondLevelInputObject'].coerce_isolated_input({})
+        assert_equal 42, result[:example][:b]
+        assert_equal "Default", result[:example][:c]
+        assert_equal true, result[:example][:d]
+      end
+    end
+
+    describe "when sent into a query" do
+      let(:variables) { {} }
+      let(:result) { Dummy::Schema.execute(query_string, variables: variables) }
+
+      describe "list inputs" do
+        let(:variables) { {"search" => [MinimumInputObject.new({"source" => "COW", "fatContent" => 0.4})]} }
+        let(:query_string) {%|
+          query getCheeses($search: [DairyProductInput]!){
+              sheep: searchDairy(product: [{source: SHEEP, fatContent: 0.1}]) {
+                ... cheeseFields
+              }
+              cow: searchDairy(product: $search) {
+                ... cheeseFields
+              }
+          }
+
+          fragment cheeseFields on Cheese {
+            flavor
+          }
+        |}
+
+        it "converts items to plain values" do
+          sheep_value = result["data"]["sheep"]["flavor"]
+          cow_value = result["data"]["cow"]["flavor"]
+          assert_equal("Manchego", sheep_value)
+          assert_equal("Brie", cow_value)
+        end
+      end
+    end
+  end
+
+  describe "failed loads on input object arguments" do
+    class FailedLoadSchema < GraphQL::Schema
+      class Thing < GraphQL::Schema::Object
+        field :id, ID
+      end
+
+      class ThingInput < GraphQL::Schema::InputObject
+        argument :id, ID, loads: Thing, as: :thing
+      end
+
+      class GetThings < GraphQL::Schema::Resolver
+        argument :things, [ThingInput]
+
+        type [Thing], null: false
+        def resolve(things:)
+          things.map { |t| t[:thing]}
+        end
+      end
+
+      class Query < GraphQL::Schema::Object
+        field :get_things, resolver: GetThings
+      end
+
+      query(Query)
+
+      def self.object_from_id(id, ctx)
+        -> { nil }
+      end
+
+      lazy_resolve(Proc, :call)
+
+      rescue_from(StandardError) {
+        nil
+      }
+    end
+
+    it "handles a lazy failed load of an argument with a nice error" do
+      res = FailedLoadSchema.execute("{ getThings(things: [{id: \"1\"}]) { id } }")
+      assert_equal ["No object found for `id: \"1\"`"], res["errors"].map { |e| e["message"] }
+    end
+  end
 end
