@@ -16,7 +16,7 @@ module InstrumentationSpec
   end
 
   class Query < GraphQL::Schema::Object
-    field :some_field, [SomeInterface], null: true
+    field :some_field, [SomeInterface]
 
     def some_field
       nil
@@ -26,10 +26,6 @@ module InstrumentationSpec
   class Schema < GraphQL::Schema
     query Query
     orphan_types [SomeType]
-    if TESTING_INTERPRETER
-      use GraphQL::Execution::Interpreter
-      use GraphQL::Analysis::AST
-    end
   end
 end
 
