@@ -64,24 +64,16 @@ describe GraphQL::Schema::Member::Scoped do
         scope: false,
         resolver_method: :items
 
-      field :nil_items, [Item], null: true
+      field :nil_items, [Item]
       def nil_items
         nil
       end
 
       field :french_items, [FrenchItem], null: false,
         resolver_method: :items
-      if TESTING_INTERPRETER
-        field :items_connection, Item.connection_type, null: false,
+
+      field :items_connection, Item.connection_type, null: false,
           resolver_method: :items
-      else
-        field :items_connection, Item.connection_type, null: false, resolve: ->(obj, args, ctx) {
-          [
-            OpenStruct.new(name: "Trombone"),
-            OpenStruct.new(name: "Paperclip"),
-          ]
-        }
-      end
 
       def items
         [
@@ -90,34 +82,19 @@ describe GraphQL::Schema::Member::Scoped do
         ]
       end
 
+      field :things, [Thing], null: false
+      def things
+        items + [OpenStruct.new(name: "Turbine")]
+      end
+
       field :lazy_items, [Item], null: false
       field :lazy_items_connection, Item.connection_type, null: false, resolver_method: :lazy_items
       def lazy_items
         ->() { items }
       end
-
-      if TESTING_INTERPRETER
-        field :things, [Thing], null: false
-        def things
-          items + [OpenStruct.new(name: "Turbine")]
-        end
-      else
-        # Make sure it works with resolve procs, too
-        field :things, [Thing], null: false, resolve: ->(obj, args, ctx) {
-          [
-            OpenStruct.new(name: "Trombone"),
-            OpenStruct.new(name: "Paperclip"),
-            OpenStruct.new(name: "Turbine"),
-          ]
-        }
-      end
     end
 
     query(Query)
-    if TESTING_INTERPRETER
-      use GraphQL::Execution::Interpreter
-      use GraphQL::Analysis::AST
-    end
     lazy_resolve(Proc, :call)
   end
 
