@@ -34,14 +34,15 @@ describe GraphQL::Schema::Subscription do
     end
 
     class TootWasTooted < BaseSubscription
-      argument :handle, String, required: true, loads: User, as: :user, camelize: false
+      argument :handle, String, loads: User, as: :user, camelize: false
 
       field :toot, Toot, null: false
       field :user, User, null: false
       # Can't subscribe to private users
       def authorized?(user:, path:, query:)
         if user[:private]
-          raise GraphQL::ExecutionError, "Can't subscribe to private user (#{path})"
+          context[:last_path] = path
+          false
         else
           true
         end
@@ -60,7 +61,7 @@ describe GraphQL::Schema::Subscription do
         if context[:viewer] == user
           # don't update for one's own toots.
           # (IRL it would make more sense to implement this in `#subscribe`)
-          :no_update
+          NO_UPDATE
         else
           # This assumes that trigger object can fulfill `{toot:, user:}`,
           # for testing that the default implementation is `return object`
@@ -75,10 +76,14 @@ describe GraphQL::Schema::Subscription do
       field :user, User, null: false
     end
 
+    class DirectTootWasTootedWithOptionalScope < DirectTootWasTooted
+      subscription_scope :viewer, optional: true
+    end
+
     # Test initial response, which returns all users
     class UsersJoined < BaseSubscription
       class UsersJoinedManualPayload < GraphQL::Schema::Object
-        field :users, [User], null: true,
+        field :users, [User],
           description: "Includes newly-created users, or all users on the initial load"
       end
 
@@ -97,20 +102,21 @@ describe GraphQL::Schema::Subscription do
     # Like above, but doesn't override #subscription,
     # to make sure it works without arguments
     class NewUsersJoined < BaseSubscription
-      field :users, [User], null: true,
+      field :users, [User],
         description: "Includes newly-created users, or all users on the initial load"
     end
 
     class Subscription < GraphQL::Schema::Object
       field :toot_was_tooted, subscription: TootWasTooted, extras: [:path, :query]
       field :direct_toot_was_tooted, subscription: DirectTootWasTooted
+      field :direct_toot_was_tooted_with_optional_scope, subscription: DirectTootWasTootedWithOptionalScope
       field :users_joined, subscription: UsersJoined
       field :new_users_joined, subscription: NewUsersJoined
     end
 
     class Mutation < GraphQL::Schema::Object
       field :toot, Toot, null: false do
-        argument :body, String, required: true
+        argument :body, String
       end
 
       def toot(body:)
@@ -124,18 +130,25 @@ describe GraphQL::Schema::Subscription do
     query(Query)
     mutation(Mutation)
     subscription(Subscription)
-    use GraphQL::Execution::Interpreter
-    use GraphQL::Analysis::AST
-    use GraphQL::Execution::Errors
 
     rescue_from(StandardError) { |err, *rest|
-      raise "This should never happen: #{err.class}: #{err.message}"
+      if err.is_a?(GraphQL::Subscriptions::SubscriptionScopeMissingError)
+        raise err
+      else
+        err2 = RuntimeError.new("This should never happen: #{err.class}: #{err.message}")
+        err2.set_backtrace(err.backtrace)
+        raise err2
+      end
     }
 
     def self.object_from_id(id, ctx)
       USERS[id]
     end
 
+    def self.unauthorized_field(err)
+      path = err.context[:last_path]
+      raise GraphQL::ExecutionError, "Can't subscribe to private user (#{path})"
+    end
 
     class InMemorySubscriptions < GraphQL::Subscriptions
       SUBSCRIPTION_REGISTRY = {}
@@ -151,9 +164,9 @@ describe GraphQL::Schema::Subscription do
         SUBSCRIPTION_REGISTRY[subscription_id] = [query, events]
       end
 
-      def each_subscription_id(event)
+      def execute_all(event, object)
         EVENT_REGISTRY[event.topic].each do |sub_id|
-          yield(sub_id)
+          execute(sub_id, event, object)
         end
       end
 
@@ -380,7 +393,7 @@ describe GraphQL::Schema::Subscription do
       assert_equal [{"handle" => "eileencodes"}, {"handle" => "tenderlove"}], update["data"]["usersJoined"]["users"]
     end
 
-    it "skips the update if `:no_update` is returned, but updates other subscribers" do
+    it "skips the update if `NO_UPDATE` is returned, but updates other subscribers" do
       query_str = <<-GRAPHQL
       subscription {
         tootWasTooted(handle: "matz") {
@@ -402,7 +415,7 @@ describe GraphQL::Schema::Subscription do
       assert_equal "Merry Christmas, here's a new Ruby version", mailbox1.first["data"]["tootWasTooted"]["toot"]["body"]
       # But not matz:
       assert_equal [], mailbox2
-      # `:no_update` doesn't cause an unsubscribe
+      # `NO_UPDATE` doesn't cause an unsubscribe
       assert_equal 2, in_memory_subscription_count
     end
 
@@ -481,6 +494,66 @@ describe GraphQL::Schema::Subscription do
       assert_equal :viewer, scoped_subscription.subscription_scope
     end
 
+    it "requires subscription scope for subscribe and update by default" do
+      err = assert_raises GraphQL::Subscriptions::SubscriptionScopeMissingError do
+        exec_query <<-GRAPHQL
+          subscription {
+            directTootWasTooted {
+              toot { body }
+            }
+          }
+        GRAPHQL
+      end
+      expected_message = "Subscription.directTootWasTooted (SubscriptionFieldSchema::DirectTootWasTooted) requires a `scope:` value to trigger updates (Set `subscription_scope ..., optional: true` to disable this requirement)"
+      assert_equal expected_message, err.message
+      assert_equal 0, in_memory_subscription_count
+
+      exec_query <<-GRAPHQL, context: { viewer: :me }
+        subscription {
+          directTootWasTooted {
+            toot { body }
+          }
+        }
+      GRAPHQL
+      assert_equal 1, in_memory_subscription_count
+
+      obj = OpenStruct.new(toot: { body: "Hello from matz!" }, user: SubscriptionFieldSchema::USERS["matz"])
+      err = assert_raises GraphQL::Subscriptions::SubscriptionScopeMissingError do
+        SubscriptionFieldSchema.subscriptions.trigger(:direct_toot_was_tooted, {}, obj)
+      end
+      assert_equal expected_message, err.message
+    end
+
+    it "doesn't require subscription scope if `optional: true`" do
+      res = exec_query <<-GRAPHQL, context: { viewer: :me }
+        subscription {
+          directTootWasTootedWithOptionalScope {
+            toot { body }
+          }
+        }
+      GRAPHQL
+
+      res2 = exec_query <<-GRAPHQL
+        subscription {
+          directTootWasTootedWithOptionalScope {
+            toot { body }
+          }
+        }
+      GRAPHQL
+
+      assert_equal 2, in_memory_subscription_count
+
+      obj = OpenStruct.new(toot: { body: "Hello from matz!" }, user: SubscriptionFieldSchema::USERS["matz"])
+
+      SubscriptionFieldSchema.subscriptions.trigger(:direct_toot_was_tooted_with_optional_scope, {}, obj)
+      assert_equal 0, res.context[:subscription_mailbox].length
+      assert_equal 1, res2.context[:subscription_mailbox].length
+
+      SubscriptionFieldSchema.subscriptions.trigger(:direct_toot_was_tooted_with_optional_scope, {}, obj, scope: :me)
+      assert_equal 1, res.context[:subscription_mailbox].length
+      assert_equal 1, res2.context[:subscription_mailbox].length
+    end
+
     it "provides a subscription scope that is used in execution" do
       res = exec_query <<-GRAPHQL, context: { viewer: :me }
         subscription {
