@@ -106,9 +106,9 @@ module MaskHelpers
 
   class WithinInputType < BaseInputObject
     metadata :hidden_input_object_type, true
-    argument :latitude, Float, required: true
-    argument :longitude, Float, required: true
-    argument :miles, Float, required: true do
+    argument :latitude, Float
+    argument :longitude, Float
+    argument :miles, Float do
       metadata :hidden_input_field, true
     end
   end
@@ -134,9 +134,9 @@ module MaskHelpers
       end
     end
 
-    field :language, LanguageType, null: true do
+    field :language, LanguageType do
       metadata :hidden_field, true
-      argument :name, String, required: true do
+      argument :name, String do
         metadata :hidden_argument, true
       end
     end
@@ -149,25 +149,25 @@ module MaskHelpers
       argument :manners, [MannerType], required: false, description: "Filter phonemes by manner of articulation"
     end
 
-    field :phoneme, PhonemeType, null: true do
+    field :phoneme, PhonemeType do
       description "Lookup a phoneme by symbol"
-      argument :symbol, String, required: true
+      argument :symbol, String
     end
 
-    field :unit, EmicUnitType, null: true do
+    field :unit, EmicUnitType do
       description "Find an emic unit by its name"
-      argument :name, String, required: true
+      argument :name, String
     end
 
     field :manners, [MannerType], null: false
   end
 
   class MutationType < BaseObject
-    field :add_phoneme, PhonemeType, null: true do
+    field :add_phoneme, PhonemeType do
       argument :symbol, String, required: false
     end
 
-    field :add_chereme, String, null: true do
+    field :add_chereme, String do
       argument :chereme, CheremeInput, required: false do
         metadata :hidden_argument, true
       end
@@ -197,10 +197,6 @@ module MaskHelpers
     end
 
     instrument :query, FilterInstrumentation
-    if TESTING_INTERPRETER
-      use GraphQL::Execution::Interpreter
-      use GraphQL::Analysis::AST
-    end
   end
 
   module Data
@@ -254,7 +250,7 @@ describe GraphQL::Schema::Warden do
   end
 
   describe "hiding root types" do
-    let(:mask) { ->(m, ctx) { TESTING_INTERPRETER ? m == MaskHelpers::MutationType : m == MaskHelpers::MutationType.graphql_definition  } }
+    let(:mask) { ->(m, ctx) { m == MaskHelpers::MutationType } }
 
     it "acts as if the root doesn't exist" do
       query_string = %|mutation { addPhoneme(symbol: "ϕ") { name } }|
@@ -448,6 +444,7 @@ describe GraphQL::Schema::Warden do
         class BagOfThings < GraphQL::Schema::Union
           possible_types A, B, C
         end
+
         class Query < GraphQL::Schema::Object
           field :bag, BagOfThings, null: false
         end
@@ -469,17 +466,17 @@ describe GraphQL::Schema::Warden do
       assert_equal ["bag"], res["data"]["Query"]["fields"].map { |f| f["name"] }
 
       # Hide the union when all its possible types are gone. This will cause the field to be hidden too.
-      res = schema.execute(query_string, except: ->(m, _) { ["A", "B", "C"].include?(m.name) })
+      res = schema.execute(query_string, except: ->(m, _) { ["A", "B", "C"].include?(m.graphql_name) })
       assert_nil res["data"]["BagOfThings"]
       assert_equal [], res["data"]["Query"]["fields"]
 
-      res = schema.execute(query_string, except: ->(m, _) { m.name == "bag" })
+      res = schema.execute(query_string, except: ->(m, _) { m.graphql_name == "bag" })
       assert_nil res["data"]["BagOfThings"]
       assert_equal [], res["data"]["Query"]["fields"]
 
       # Unreferenced but still visible because orphan type
-      schema.graphql_definition.orphan_types = [schema.find("BagOfThings").graphql_definition]
-      res = schema.execute(query_string, except: ->(m, _) { m.name == "bag" })
+      schema.orphan_types([schema.find("BagOfThings")])
+      res = schema.execute(query_string, except: ->(m, _) { m.graphql_name == "bag" })
       assert res["data"]["BagOfThings"]
     end
 
