@@ -15,6 +15,7 @@ describe GraphQL::StaticValidation::UniqueDirectivesPerLocation do
 
     directive @A on FIELD
     directive @B on FIELD
+    directive @C repeatable on FIELD
   ") }
 
   describe "query with no directives" do
@@ -31,6 +32,20 @@ describe GraphQL::StaticValidation::UniqueDirectivesPerLocation do
     end
   end
 
+  describe "query with repeatable directives" do
+    let(:query_string) {"
+      {
+        type {
+          field @C @C @C
+        }
+      }
+    "}
+
+    it "passes rule" do
+      assert_equal [], errors
+    end
+  end
+
   describe "query with unique directives in different locations" do
     let(:query_string) {"
       {
