@@ -23,14 +23,6 @@ class InMemoryBackend
       end
     end
 
-    def each_subscription_id(event)
-      @events[event.topic].each do |fp, sub_ids|
-        sub_ids.each do |sub_id|
-          yield(sub_id)
-        end
-      end
-    end
-
     def read_subscription(subscription_id)
       query = @queries[subscription_id]
       if query
@@ -66,8 +58,10 @@ class InMemoryBackend
       sub_ids_by_fp = @events[topic]
       sub_ids_by_fp.each do |fingerprint, sub_ids|
         result = execute_update(sub_ids.first, event, object)
-        sub_ids.each do |sub_id|
-          deliver(sub_id, result)
+        if !result.nil?
+          sub_ids.each do |sub_id|
+            deliver(sub_id, result)
+          end
         end
       end
     end
@@ -94,6 +88,7 @@ class InMemoryBackend
       @pushes.clear
     end
   end
+
   # Just a random stateful object for tracking what happens:
   class SubscriptionPayload
     attr_reader :str
@@ -124,58 +119,69 @@ class ClassBasedInMemoryBackend < InMemoryBackend
   end
 
   class StreamInput < GraphQL::Schema::InputObject
-    argument :user_id, ID, required: true, camelize: false
+    argument :user_id, ID, camelize: false
     argument :payload_type, PayloadType, required: false, default_value: "ONE", prepare: ->(e, ctx) { e ? e.downcase : e }
   end
 
   class EventSubscription < GraphQL::Schema::Subscription
-    argument :user_id, ID, required: true
+    argument :user_id, ID
     argument :payload_type, PayloadType, required: false, default_value: "ONE", prepare: ->(e, ctx) { e ? e.downcase : e }
-    field :payload, Payload, null: true
+    field :payload, Payload
+  end
+
+  class FilteredStream < GraphQL::Schema::Subscription
+    subscription_scope :segment
+    argument :channel, Integer, required: false
+
+    field :message, String, null: false
+
+    def update(channel: nil)
+      if channel && object.channel != channel
+        NO_UPDATE
+      else
+        super
+      end
+    end
+
+    def self.topic_for(arguments:, field:, scope:)
+      "#{field.graphql_name}:#{scope}"
+    end
   end
 
   class Subscription < GraphQL::Schema::Object
-    if !TESTING_INTERPRETER
-      # Stub methods are required
-      [:payload, :event, :my_event].each do |m|
-        define_method(m) { |*a| nil }
-      end
-    end
     field :payload, Payload, null: false do
-      argument :id, ID, required: true
+      argument :id, ID
     end
 
-    field :event, Payload, null: true do
+    field :event, Payload do
       argument :stream, StreamInput, required: false
     end
 
     field :event_subscription, subscription: EventSubscription
 
-    field :my_event, Payload, null: true, subscription_scope: :me do
+    field :my_event, Payload, subscription_scope: :me do
       argument :payload_type, PayloadType, required: false
     end
 
     field :failed_event, Payload, null: false  do
-      argument :id, ID, required: true
+      argument :id, ID
     end
 
     def failed_event(id:)
       raise GraphQL::ExecutionError.new("unauthorized")
     end
+
+    field :filtered_stream, subscription: FilteredStream
   end
 
   class Query < GraphQL::Schema::Object
-    field :dummy, Integer, null: true
+    field :dummy, Integer
   end
 
   class Schema < GraphQL::Schema
     query(Query)
     subscription(Subscription)
     use InMemoryBackend::Subscriptions, extra: 123
-    if TESTING_INTERPRETER
-      use GraphQL::Execution::Interpreter
-      use GraphQL::Analysis::AST
-    end
   end
 end
 
@@ -233,7 +239,7 @@ class FromDefinitionInMemoryBackend < InMemoryBackend
       "failedEvent" => ->(o,a,c) { raise GraphQL::ExecutionError.new("unauthorized") },
     },
   }
-  Schema = GraphQL::Schema.from_definition(SchemaDefinition, default_resolve: Resolvers, using: {InMemoryBackend::Subscriptions => { extra: 123 }}, interpreter: TESTING_INTERPRETER)
+  Schema = GraphQL::Schema.from_definition(SchemaDefinition, default_resolve: Resolvers, using: {InMemoryBackend::Subscriptions => { extra: 123 }})
   # TODO don't hack this (no way to add metadata from IDL parser right now)
   Schema.get_field("Subscription", "myEvent").subscription_scope = :me
 end
@@ -283,7 +289,7 @@ describe GraphQL::Subscriptions do
           res_1 = schema.execute(query_str, context: { socket: "1" }, variables: { "id" => "100" }, root_value: root_object)
           res_2 = schema.execute(query_str, context: { socket: "2" }, variables: { "id" => "200" }, root_value: root_object)
 
-          empty_response = TESTING_INTERPRETER ? {} : nil
+          empty_response = {}
 
           # Initial response is nil, no broadcasts yet
           assert_equal(empty_response, res_1["data"])
@@ -306,6 +312,46 @@ describe GraphQL::Subscriptions do
         end
       end
 
+      if in_memory_backend_class != FromDefinitionInMemoryBackend # No way to specify this when using IDL
+        it "supports filtering in the subscription class" do
+          query_str = "subscription($channel: Int) { filteredStream(channel: $channel) { message } }"
+
+          # Unfiltered:
+          schema.execute(query_str, context: { socket: "1", segment: "A" }, variables: {})
+          # Filtered:
+          schema.execute(query_str, context: { socket: "2", segment: "A" }, variables: { channel: 1 })
+          schema.execute(query_str, context: { socket: "3", segment: "A" }, variables: { channel: 2 })
+
+          # Another Subscription scope:
+          schema.execute(query_str, context: { socket: "4", segment: "B" }, variables: {})
+          schema.execute(query_str, context: { socket: "5", segment: "B" }, variables: { channel: 1 })
+
+          schema.subscriptions.trigger(:filtered_stream, {}, OpenStruct.new(channel: 1, message: "Message 1"), scope: "A")
+          schema.subscriptions.trigger(:filtered_stream, {}, OpenStruct.new(channel: 2, message: "Message 2"), scope: "A")
+          schema.subscriptions.trigger(:filtered_stream, {}, OpenStruct.new(channel: 3, message: "Message 3"), scope: "A")
+
+          # Unfiltered, received all updates:
+          assert_equal 3, deliveries["1"].size
+          # Only received updates that matched `channel`:
+          assert_equal 1, deliveries["2"].size
+          assert_equal 1, deliveries["3"].size
+          # Different segment, no updates:
+          assert_equal 0, deliveries["4"].size
+          assert_equal 0, deliveries["5"].size
+
+          schema.subscriptions.trigger(:filtered_stream, {}, OpenStruct.new(channel: 1, message: "Message 4"), scope: "B")
+          schema.subscriptions.trigger(:filtered_stream, {}, OpenStruct.new(channel: 2, message: "Message 5"), scope: "B")
+
+          # These should be unchanged because the later triggers had a different scope value:
+          assert_equal 3, deliveries["1"].size
+          assert_equal 1, deliveries["2"].size
+          assert_equal 1, deliveries["3"].size
+          # These received updates from the second set of triggers:
+          assert_equal 2, deliveries["4"].size
+          assert_equal 1, deliveries["5"].size
+        end
+      end
+
       it "sends updated data for multifield subscriptions" do
         query_str = <<-GRAPHQL
         subscription ($id: ID!){
@@ -316,7 +362,7 @@ describe GraphQL::Subscriptions do
 
         # Initial subscriptions
         res = schema.execute(query_str, context: { socket: "1" }, variables: { "id" => "100" }, root_value: root_object)
-        empty_response = TESTING_INTERPRETER ? {} : nil
+        empty_response = {}
 
         # Initial response is nil, no broadcasts yet
         assert_equal(empty_response, res["data"])
@@ -355,7 +401,7 @@ describe GraphQL::Subscriptions do
           # Initial subscriptions
           response = schema.execute(nil, document: document, context: { socket: "1" }, variables: { "id" => "100" }, root_value: root_object)
 
-          empty_response = TESTING_INTERPRETER ? {} : nil
+          empty_response = {}
 
           # Initial response is empty, no broadcasts yet
           assert_equal(empty_response, response["data"])
@@ -581,20 +627,11 @@ describe GraphQL::Subscriptions do
 
             # There's no way to add `prepare:` when using SDL, so only the Ruby-defined schema has it
             expected_sub_count = if schema == ClassBasedInMemoryBackend::Schema
-              if TESTING_INTERPRETER
-                {
-                  ":eventSubscription:payloadType:one:userId:3" => 1,
-                  ":eventSubscription:payloadType:one:userId:4" => 2,
-                  ":eventSubscription:payloadType:two:userId:3" => 1,
-                }
-              else
-                # Unfortunately, on the non-interpreter runtime, `prepare:` was _not_ applied here,
-                {
-                  ":eventSubscription:payloadType:ONE:userId:3" => 1,
-                  ":eventSubscription:payloadType:ONE:userId:4" => 2,
-                  ":eventSubscription:payloadType:TWO:userId:3" => 1,
-                }
-              end
+              {
+                ":eventSubscription:payloadType:one:userId:3" => 1,
+                ":eventSubscription:payloadType:one:userId:4" => 2,
+                ":eventSubscription:payloadType:two:userId:3" => 1,
+              }
             else
               {
                 ":eventSubscription:payloadType:ONE:userId:3" => 1,
@@ -777,8 +814,6 @@ describe GraphQL::Subscriptions do
 
       query(Query)
       subscription(Subscription)
-      use GraphQL::Execution::Interpreter
-      use GraphQL::Analysis::AST
       use InMemoryBackend::Subscriptions, extra: nil,
         broadcast: true, default_broadcastable: true
     end
