@@ -4,13 +4,17 @@ require "spec_helper"
 describe GraphQL::Types::Relay::BaseEdge do
   module NonNullableDummy
     class NonNullableNode < GraphQL::Schema::Object
-      field :some_field, String, null: true
+      field :some_field, String
     end
 
     class NonNullableNodeEdgeType < GraphQL::Types::Relay::BaseEdge
       node_type(NonNullableNode, null: false)
     end
 
+    class NonNullableNodeClassOverrideEdgeType < GraphQL::Types::Relay::BaseEdge
+      node_nullable(false)
+    end
+
     class NonNullableNodeEdgeConnectionType < GraphQL::Types::Relay::BaseConnection
       edge_type(NonNullableNodeEdgeType, nodes_field: false)
     end
@@ -32,4 +36,19 @@ describe GraphQL::Types::Relay::BaseEdge do
     assert_equal "NON_NULL", node_field["type"]["kind"]
     assert_equal "NonNullableNode", node_field["type"]["ofType"]["name"]
   end
+
+  it "supports class-level node_nullable config" do
+    assert_equal false, NonNullableDummy::NonNullableNodeClassOverrideEdgeType.node_nullable
+  end
+
+  it "Supports extra kwargs for node field" do
+    extension = Class.new(GraphQL::Schema::FieldExtension)
+    connection = Class.new(GraphQL::Types::Relay::BaseEdge) do
+      node_type(GraphQL::Schema::Object, field_options: { extensions: [extension] })
+    end
+
+    field = connection.fields["node"]
+    assert_equal 1, field.extensions.size
+    assert_instance_of extension, field.extensions.first
+  end
 end
