@@ -4,7 +4,7 @@ require "spec_helper"
 describe GraphQL::Schema::InputObject do
   let(:input_object) do
     # Cause the arguments_class to be prepared
-    Dummy::DairyProductInput.to_graphql
+    Dummy::DairyProductInput.deprecated_to_graphql
 
     Dummy::DairyProductInput.new(
       { source: 'COW',  fatContent: 0.8 },
