@@ -4,8 +4,8 @@ require_relative "./data"
 module Dummy
   class NoSuchDairyError < StandardError; end
 
-  GraphQL::Field.accepts_definitions(joins: GraphQL::Define.assign_metadata_key(:joins))
-  GraphQL::BaseType.accepts_definitions(class_names: GraphQL::Define.assign_metadata_key(:class_names))
+  GraphQL::Field.deprecated_accepts_definitions(joins: GraphQL::Define.assign_metadata_key(:joins))
+  GraphQL::BaseType.deprecated_accepts_definitions(class_names: GraphQL::Define.assign_metadata_key(:class_names))
 
   class BaseField < GraphQL::Schema::Field
     accepts_definition :joins
@@ -50,7 +50,7 @@ module Dummy
     description "Something you can eat, yum"
     field :fat_content, Float, null: false, description: "Percentage which is fat"
     field :origin, String, null: false, description: "Place the edible comes from"
-    field :self_as_edible, Edible, null: true
+    field :self_as_edible, Edible
     def self_as_edible
       object
     end
@@ -100,10 +100,10 @@ module Dummy
       "Animal which produced the milk for this cheese",
       null: false
 
-    field :similar_cheese, Cheese, "Cheeses like this one", null: true  do
+    field :similar_cheese, Cheese, "Cheeses like this one"  do
       # metadata test
       joins [:cheeses, :milks]
-      argument :source, [DairyAnimal], required: true
+      argument :source, [DairyAnimal]
       argument :nullable_source, [DairyAnimal], required: false, default_value: [1]
     end
 
@@ -117,12 +117,12 @@ module Dummy
       end
     end
 
-    field :nullable_cheese, Cheese, "Cheeses like this one", null: true do
+    field :nullable_cheese, Cheese, "Cheeses like this one" do
       argument :source, [DairyAnimal], required: false
     end
     def nullable_cheese; raise("NotImplemented"); end
 
-    field :deeply_nullable_cheese, Cheese, "Cheeses like this one", null: true do
+    field :deeply_nullable_cheese, Cheese, "Cheeses like this one" do
       argument :source, [[DairyAnimal, null: true], null: true], required: false
     end
     def deeply_nullable_cheese; raise("NotImplemented"); end
@@ -145,7 +145,7 @@ module Dummy
     field :id, ID, null: false
     field :source, DairyAnimal, null: false, description: "Animal which produced this milk"
     field :origin, String, null: false, description: "Place the milk comes from"
-    field :flavors, [String, null: true], null: true, description: "Chocolate, Strawberry, etc" do
+    field :flavors, [String, null: true], description: "Chocolate, Strawberry, etc" do
       argument :limit, Int, required: false
     end
 
@@ -153,10 +153,10 @@ module Dummy
       limit ? object.flavors.first(limit) : object.flavors
     end
 
-    field :execution_error, String, null: true
+    field :execution_error, String
     def execution_error; raise(GraphQL::ExecutionError, "There was an execution error"); end
 
-    field :all_dairy, ["Dummy::DairyProduct", null: true], null: true
+    field :all_dairy, ["Dummy::DairyProduct", null: true]
     def all_dairy; CHEESES.values + MILKS.values; end
   end
 
@@ -169,7 +169,7 @@ module Dummy
 
   module Sweetener
     include BaseInterface
-    field :sweetness, Integer, null: true
+    field :sweetness, Integer
 
     orphan_types Aspartame
   end
@@ -177,7 +177,7 @@ module Dummy
   # No actual data; This type is an "orphan", only accessible through Interfaces
   class Honey < BaseObject
     description "Sweet, dehydrated bee barf"
-    field :flower_type, String, "What flower this honey came from", null: true
+    field :flower_type, String, "What flower this honey came from"
     implements Edible
     implements AnimalProduct
     implements Sweetener
@@ -186,7 +186,7 @@ module Dummy
   # No actual data; Same as "Honey", but only accessible through an interface's orphans
   class Aspartame < BaseObject
     description "Sugar substitute with an off-flavor aftertaste"
-    field :manufacturer, String, "What manufacturer this aspartame came from", null: true
+    field :manufacturer, String, "What manufacturer this aspartame came from"
     implements Edible
     implements Sweetener
   end
@@ -194,21 +194,21 @@ module Dummy
   class Dairy < BaseObject
     description "A farm where milk is harvested and cheese is produced"
     field :id, ID, null: false
-    field :cheese, Cheese, null: true
-    field :milks, [Milk, null: true], null: true
+    field :cheese, Cheese
+    field :milks, [Milk, null: true]
   end
 
   class MaybeNull < BaseObject
     description "An object whose fields return nil"
-    field :cheese, Cheese, null: true
+    field :cheese, Cheese
   end
 
   class TracingScalar < BaseObject
     description "An object which has traced scalars"
 
-    field :trace_nil, Integer, null: true
-    field :trace_false, Integer, null: true, trace: false
-    field :trace_true, Integer, null: true, trace: true
+    field :trace_nil, Integer
+    field :trace_false, Integer, trace: false
+    field :trace_true, Integer, trace: true
   end
 
   class DairyProduct < BaseUnion
@@ -220,8 +220,8 @@ module Dummy
   class Cow < BaseObject
     description "A bovine animal that produces milk"
     field :id, ID, null: false
-    field :name, String, null: true
-    field :last_produced_dairy, DairyProduct, null: true
+    field :name, String
+    field :last_produced_dairy, DairyProduct
 
     field :cant_be_null_but_is, String, null: false
     def cant_be_null_but_is; nil; end
@@ -233,8 +233,8 @@ module Dummy
   class Goat < BaseObject
     description "An caprinae animal that produces milk"
     field :id, ID, null: false
-    field :name, String, null: true
-    field :last_produced_dairy, DairyProduct, null: true
+    field :name, String
+    field :last_produced_dairy, DairyProduct
   end
 
   class Animal < BaseUnion
@@ -254,12 +254,12 @@ module Dummy
     graphql_name "ResourceOrderType"
     description "Properties used to determine ordering"
 
-    argument :direction, String, required: true, description: "ASC or DESC"
+    argument :direction, String, description: "ASC or DESC"
   end
 
   class DairyProductInput < BaseInputObject
     description "Properties for finding a dairy product"
-    argument :source, DairyAnimal, required: true, description: "Where it came from"
+    argument :source, DairyAnimal, description: "Where it came from"
     argument :origin_dairy, String, required: false, description: "Dairy which produced it", default_value: "Sugar Hollow Dairy"
     argument :fat_content, Float, required: false, description: "How much fat it has", default_value: 0.3
     argument :organic, Boolean, required: false, default_value: false
@@ -281,6 +281,7 @@ module Dummy
 
   class Time < BaseScalar
     description "Time since epoch in seconds"
+    specified_by_url "https://time.graphql"
 
     def self.coerce_input(value, ctx)
       Time.at(Float(value))
@@ -303,7 +304,7 @@ module Dummy
         self.data = data
         type(type, null: true)
         description("Find a #{type.name} by id")
-        argument :id, id_type, required: true
+        argument :id, id_type
       end
     end
 
@@ -328,6 +329,9 @@ module Dummy
     end
 
     def resolve
+      if context[:resolved_count]
+        context[:resolved_count] += 1
+      end
       self.class.data
     end
   end
@@ -336,14 +340,14 @@ module Dummy
     graphql_name "Query"
     description "Query root of the system"
     # Returns `root_value:`
-    field :root, String, null: true
+    field :root, String
     def root
       object
     end
     field :cheese, resolver: FetchItem.build(type: Cheese, data: CHEESES)
     field :milk, resolver: FetchItem.build(type: Milk, data: MILKS, id_type: "ID")
     field :dairy, resolver: GetSingleton.build(type: Dairy, data: DAIRY)
-    field :from_source, [Cheese, null: true], null: true, description: "Cheese from source" do
+    field :from_source, [Cheese, null: true], description: "Cheese from source" do
       argument :source, DairyAnimal, required: false, default_value: 1
       argument :old_source, String, required: false, deprecation_reason: "No longer supported"
     end
@@ -351,7 +355,7 @@ module Dummy
       CHEESES.values.select { |c| c.source == source }
     end
 
-    field :favorite_edible, Edible, null: true, description: "My favorite food"
+    field :favorite_edible, Edible, description: "My favorite food"
     def favorite_edible
       MILKS[1]
     end
@@ -361,6 +365,9 @@ module Dummy
       description "Find dairy products matching a description"
       # This is a list just for testing 😬
       argument :product, [DairyProductInput, null: true], required: false, default_value: [{source: "SHEEP"}]
+      argument :old_product, [DairyProductInput], required: false, deprecation_reason: "No longer supported"
+      argument :single_product, DairyProductInput, required: false
+      argument :product_ids, [String], required: false, deprecation_reason: "No longer supported"
       argument :expires_after, Time, required: false
     end
 
@@ -380,7 +387,7 @@ module Dummy
 
     field :all_animal_as_cow, [AnimalAsCow, null: true], null: false, resolver_method: :all_animal
 
-    field :all_dairy, [DairyProduct, null: true], null: true do
+    field :all_dairy, [DairyProduct, null: true] do
       argument :execution_error_at_index, Integer, required: false
     end
     def all_dairy(execution_error_at_index: nil)
@@ -391,19 +398,19 @@ module Dummy
       result
     end
 
-    field :all_edible, [Edible, null: true], null: true
+    field :all_edible, [Edible, null: true]
     def all_edible
       CHEESES.values + MILKS.values
     end
 
-    field :all_edible_as_milk, [EdibleAsMilk, null: true], null: true, resolver_method: :all_edible
+    field :all_edible_as_milk, [EdibleAsMilk, null: true], resolver_method: :all_edible
 
-    field :error, String, null: true, description: "Raise an error"
+    field :error, String, description: "Raise an error"
     def error
       raise("This error was raised on purpose")
     end
 
-    field :execution_error, String, null: true
+    field :execution_error, String
     def execution_error
       raise(GraphQL::ExecutionError, "There was an execution error")
     end
@@ -430,23 +437,23 @@ module Dummy
       ]
     end
 
-    field :execution_error_with_options, Integer, null: true
+    field :execution_error_with_options, Integer
     def execution_error_with_options
       GraphQL::ExecutionError.new("Permission Denied!", options: { "code" => "permission_denied" })
     end
 
-    field :execution_error_with_extensions, Integer, null: true
+    field :execution_error_with_extensions, Integer
     def execution_error_with_extensions
       GraphQL::ExecutionError.new("Permission Denied!", extensions: { code: "permission_denied" })
     end
 
     # To test possibly-null fields
-    field :maybe_null, MaybeNull, null: true
+    field :maybe_null, MaybeNull
     def maybe_null
       OpenStruct.new(cheese: nil)
     end
 
-    field :tracing_scalar, TracingScalar, null: true
+    field :tracing_scalar, TracingScalar
     def tracing_scalar
       OpenStruct.new(
         trace_nil: 2,
@@ -457,12 +464,17 @@ module Dummy
 
     field :deep_non_null, DeepNonNull, null: false
     def deep_non_null; :deep_non_null; end
+
+    field :huge_integer, Integer
+    def huge_integer
+      GraphQL::Types::Int::MAX + 1
+    end
   end
 
   class AdminDairyAppQuery < BaseObject
     field_class AdminField
 
-    field :admin_only_message, String, null: true
+    field :admin_only_message, String
     def admin_only_message
       "This field is only visible to admin"
     end
@@ -471,14 +483,14 @@ module Dummy
   GLOBAL_VALUES = []
 
   class ReplaceValuesInput < BaseInputObject
-    argument :values, [Integer], required: true, method_access: false
+    argument :values, [Integer], method_access: false
   end
 
   class DairyAppMutation < BaseObject
     graphql_name "Mutation"
     description "The root for mutations in this schema"
     field :push_value, [Integer], null: false, description: "Push a value onto a global array :D" do
-      argument :value, Integer, required: true, as: :val
+      argument :value, Integer, as: :val
     end
     def push_value(val:)
       GLOBAL_VALUES << val
@@ -486,7 +498,7 @@ module Dummy
     end
 
     field :replace_values, [Integer], "Replace the global array with new values", null: false do
-      argument :input, ReplaceValuesInput, required: true
+      argument :input, ReplaceValuesInput
     end
 
     def replace_values(input:)
@@ -497,7 +509,7 @@ module Dummy
   end
 
   class Subscription < BaseObject
-    field :test, String, null: true
+    field :test, String
     def test; "Test"; end
   end
 
@@ -525,9 +537,8 @@ module Dummy
       end
     end
 
-    use GraphQL::Execution::Interpreter
-    use GraphQL::Analysis::AST
-    use GraphQL::Execution::Errors
+    use GraphQL::Dataloader
+
     lazy_resolve(Proc, :call)
   end
 
