@@ -51,8 +51,13 @@ module Jazz
       super(*args, **kwargs)
     end
 
-    def to_graphql
-      arg_defn = super
+    def to_graphql(silence_deprecation_warning: true)
+      arg_defn = case method(:to_graphql).super_method.parameters.size
+      when 1
+        super(silence_deprecation_warning: true)
+      else
+        super()
+      end
       arg_defn.metadata[:custom] = @custom
       arg_defn
     end
@@ -101,7 +106,7 @@ module Jazz
       end
 
       def to_graphql
-        type_defn = super
+        type_defn = super(silence_deprecation_warning: true)
         configs.each do |k, v|
           type_defn.metadata[k] = v
         end
@@ -130,7 +135,12 @@ module Jazz
     end
 
     def to_graphql
-      enum_value_defn = super
+      enum_value_defn = case method(:to_graphql).super_method.arity
+      when 0
+        super
+      else
+        super(silence_deprecation_warning: true)
+      end
       enum_value_defn.metadata[:custom_setting] = @custom_setting
       enum_value_defn
     end
@@ -167,9 +177,15 @@ module Jazz
     end
   end
 
+  module WithNameField
+    def self.prepended(base)
+      base.field :name, String, null: false
+    end
+  end
+
   module NamedEntity
     include BaseInterface
-    field :name, String, null: false
+    prepend WithNameField
   end
 
   class PrivateMembership < GraphQL::Schema::TypeMembership
@@ -232,7 +248,7 @@ module Jazz
     implements PrivateNameEntity, visibility: { private: true }
     description "A group of musicians playing together"
     config :config, :configged
-    field :formed_at, String, null: true, hash_key: "formedAtDate"
+    field :formed_at, String, hash_key: "formedAtDate"
 
     # This overrides the visibility from PrivateNameEntity
     field :overridden_name, String, null: false
@@ -296,7 +312,7 @@ module Jazz
     field :instrument, InstrumentType, null: false do
       description "An object played in order to produce music"
     end
-    field :favorite_key, Key, null: true
+    field :favorite_key, Key
     # Test lists with nullable members:
     field :inspect_context, [String, null: true], null: false
     field :add_error, String, null: false, extras: [:execution_errors]
@@ -326,12 +342,16 @@ module Jazz
 
   # Since this is not a legacy input type, this test can be removed
   class LegacyInputType < GraphQL::Schema::InputObject
-    argument :int_value, Int, required: true
+    argument :int_value, Int
+  end
+
+  class FullyOptionalInput < GraphQL::Schema::InputObject
+    argument :optional_value, String, required: false
   end
 
   class InspectableInput < GraphQL::Schema::InputObject
     argument :ensemble_id, ID, required: false, loads: Ensemble
-    argument :string_value, String, required: true, description: "Test description kwarg"
+    argument :string_value, String, description: "Test description kwarg"
     argument :nested_input, InspectableInput, required: false
     argument :legacy_input, LegacyInputType, required: false
 
@@ -389,23 +409,23 @@ module Jazz
   end
 
   class CamelizedBooleanInput <  GraphQL::Schema::InputObject
-    argument :camelized_boolean, Boolean, required: true
+    argument :camelized_boolean, Boolean
   end
 
   # Another new-style definition, with method overrides
   class Query < BaseObject
     field :ensembles, [Ensemble], null: false
-    field :find, GloballyIdentifiableType, null: true do
-      argument :id, ID, required: true
+    field :find, GloballyIdentifiableType do
+      argument :id, ID
     end
     field :instruments, [InstrumentType], null: false do
       argument :family, Family, required: false
     end
     field :inspect_input, [String], null: false do
-      argument :input, InspectableInput, required: true, custom: :ok
+      argument :input, InspectableInput, custom: :ok
     end
     field :inspect_key, InspectableKey, null: false do
-      argument :key, Key, required: true
+      argument :key, Key
     end
     field :now_playing, PerformingAct, null: false
 
@@ -418,14 +438,14 @@ module Jazz
     field :hashy_ensemble, Ensemble, null: false
 
     field :echo_json, GraphQL::Types::JSON, null: false do
-      argument :input, GraphQL::Types::JSON, required: true
+      argument :input, GraphQL::Types::JSON
     end
 
     field :echo_first_json, GraphQL::Types::JSON, null: false do
-      argument :input, [GraphQL::Types::JSON], required: true
+      argument :input, [GraphQL::Types::JSON]
     end
 
-    field :upcase_check_1, String, null: true, resolver_method: :upcase_check, extras: [:upcase]
+    field :upcase_check_1, String, resolver_method: :upcase_check, extras: [:upcase]
     field :upcase_check_2, String, null: false, upcase: false, resolver_method: :upcase_check, extras: [:upcase]
     field :upcase_check_3, String, null: false, upcase: true, resolver_method: :upcase_check, extras: [:upcase]
     field :upcase_check_4, String, null: false, upcase: "why not?", resolver_method: :upcase_check, extras: [:upcase]
@@ -434,7 +454,7 @@ module Jazz
     end
 
     field :input_object_camelization, String, null: false do
-      argument :input, CamelizedBooleanInput, required: true
+      argument :input, CamelizedBooleanInput
     end
 
     def input_object_camelization(input:)
@@ -541,13 +561,13 @@ module Jazz
     end
 
     field :default_value_test_2, String, null: false, resolver_method: :default_value_test do
-      argument :arg_with_default, InspectableInput, required: false, default_value: {}
+      argument :arg_with_default, FullyOptionalInput, required: false, default_value: {}
     end
 
     field :complex_hash_key, String, null: false, hash_key: :'foo bar/fizz-buzz'
 
 
-    field :nullable_ensemble, Ensemble, null: true do
+    field :nullable_ensemble, Ensemble do
       argument :ensemble_id, ID, required: false, loads: Ensemble
     end
 
@@ -557,15 +577,15 @@ module Jazz
   end
 
   class EnsembleInput < GraphQL::Schema::InputObject
-    argument :name, String, required: true
+    argument :name, String
   end
 
   class AddInstrument < GraphQL::Schema::Mutation
     null true
     description "Register a new musical instrument in the database"
 
-    argument :name, String, required: true
-    argument :family, Family, required: true
+    argument :name, String
+    argument :family, Family
 
     field :instrument, InstrumentType, null: false
     # This is meaningless, but it's to test the conflict with `Hash#entries`
@@ -583,7 +603,7 @@ module Jazz
   end
 
   class AddEnsembleRelay < GraphQL::Schema::RelayClassicMutation
-    argument :ensemble, EnsembleInput, required: true
+    argument :ensemble, EnsembleInput
     field :ensemble, Ensemble, null: false
 
     def resolve(ensemble:)
@@ -613,7 +633,7 @@ module Jazz
     extras [:ast_node]
 
     field :node_class, String, null: false
-    field :int, Integer, null: true
+    field :int, Integer
 
     def resolve(int: nil, ast_node:)
       {
@@ -630,7 +650,7 @@ module Jazz
     argument :int, Integer, required: false
 
     field :lookahead_class, String, null: false
-    field :int, Integer, null: true
+    field :int, Integer
 
     def resolve(int: nil, lookahead:)
       {
@@ -662,10 +682,9 @@ module Jazz
     end
   end
 
-
   class RenameNamedEntity < GraphQL::Schema::RelayClassicMutation
-    argument :named_entity_id, ID, required: true, loads: NamedEntity
-    argument :new_name, String, required: true
+    argument :named_entity_id, ID, loads: NamedEntity
+    argument :new_name, String
 
     field :named_entity, NamedEntity, null: false
 
@@ -681,8 +700,8 @@ module Jazz
   end
 
   class RenamePerformingAct < GraphQL::Schema::RelayClassicMutation
-    argument :performing_act_id, ID, required: true, loads: PerformingAct
-    argument :new_name, String, required: true
+    argument :performing_act_id, ID, loads: PerformingAct
+    argument :new_name, String
 
     field :performing_act, PerformingAct, null: false
 
@@ -698,8 +717,8 @@ module Jazz
   end
 
   class RenameEnsemble < GraphQL::Schema::RelayClassicMutation
-    argument :ensemble_id, ID, required: true, loads: Ensemble
-    argument :new_name, String, required: true
+    argument :ensemble_id, ID, loads: Ensemble
+    argument :new_name, String
 
     field :ensemble, Ensemble, null: false
 
@@ -714,7 +733,7 @@ module Jazz
   end
 
   class UpvoteEnsembles < GraphQL::Schema::RelayClassicMutation
-    argument :ensemble_ids, [ID], required: true, loads: Ensemble
+    argument :ensemble_ids, [ID], loads: Ensemble
 
     field :ensembles, [Ensemble], null: false
 
@@ -726,7 +745,7 @@ module Jazz
   end
 
   class UpvoteEnsemblesAsBands < GraphQL::Schema::RelayClassicMutation
-    argument :ensemble_ids, [ID], required: true, loads: Ensemble, as: :bands
+    argument :ensemble_ids, [ID], loads: Ensemble, as: :bands
 
     field :ensembles, [Ensemble], null: false
 
@@ -738,7 +757,7 @@ module Jazz
   end
 
   class UpvoteEnsemblesIds < GraphQL::Schema::RelayClassicMutation
-    argument :ensembles_ids, [ID], required: true, loads: Ensemble
+    argument :ensembles_ids, [ID], loads: Ensemble
 
     field :ensembles, [Ensemble], null: false
 
@@ -750,8 +769,9 @@ module Jazz
   end
 
   class RenameEnsembleAsBand < RenameEnsemble
-    argument :ensemble_id, ID, required: true, loads: Ensemble, as: :band
-
+    argument :ensemble_id, ID, loads: Ensemble, as: :band
+    # This is duplicate to the inherited one; make sure it overrides it
+    field :ensemble, Ensemble, null: false
     def resolve(band:, new_name:)
       super(ensemble: band, new_name: new_name)
     end
@@ -759,7 +779,7 @@ module Jazz
 
   class LoadAndReturnEnsemble < GraphQL::Schema::RelayClassicMutation
     argument :ensemble_id, ID, required: false, loads: Ensemble
-    field :ensemble, Ensemble, null: true
+    field :ensemble, Ensemble
 
     def resolve(ensemble: nil)
       { ensemble: ensemble }
@@ -769,7 +789,7 @@ module Jazz
   class DummyOutput < GraphQL::Schema::Object
     graphql_name "DummyOutput"
 
-    field :name, String, null: true
+    field :name, String
   end
 
   class ReturnsMultipleErrors < GraphQL::Schema::Mutation
@@ -793,7 +813,7 @@ module Jazz
 
   class Mutation < BaseObject
     field :add_ensemble, Ensemble, null: false do
-      argument :input, EnsembleInput, required: true
+      argument :input, EnsembleInput
     end
 
     field :add_instrument, mutation: AddInstrument
@@ -820,7 +840,7 @@ module Jazz
     end
 
     field :prepare_input, Integer, null: false do
-      argument :input, Integer, required: true, prepare: :square, as: :squared_input
+      argument :input, Integer, prepare: :square, as: :squared_input
     end
 
     def prepare_input(squared_input:)
@@ -932,10 +952,7 @@ module Jazz
       GloballyIdentifiableType.find(id)
     end
 
-    if TESTING_INTERPRETER
-      use GraphQL::Execution::Interpreter
-      use GraphQL::Analysis::AST
-    end
+    use GraphQL::Dataloader
 
     use MetadataPlugin, value: "xyz"
   end
@@ -945,10 +962,7 @@ module Jazz
 
     disable_introspection_entry_points
 
-    if TESTING_INTERPRETER
-      use GraphQL::Execution::Interpreter
-      use GraphQL::Analysis::AST
-    end
+    use GraphQL::Dataloader
   end
 
   class SchemaWithoutSchemaIntrospection < GraphQL::Schema
