@@ -6,7 +6,7 @@ module StarWars
   class Ship < GraphQL::Schema::Object
     implements GraphQL::Types::Relay::Node
     global_id_field :id
-    field :name, String, null: true
+    field :name, String
     # Test cyclical connection types:
     field :ships, Ship.connection_type, null: false
   end
@@ -25,10 +25,9 @@ module StarWars
         end
       }
     end
-    field :planet, String, null: true
+    field :planet, String
   end
 
-
   class BaseEdge < GraphQL::Types::Relay::BaseEdge
     node_type(BaseType)
   end
@@ -45,24 +44,10 @@ module StarWars
     edge_type(BaseEdge, nodes_field: false)
     nodes_field
 
-    field :total_count, Integer, null: true
+    field :total_count, Integer
 
     def total_count
-      if TESTING_INTERPRETER
-        object.items.count
-      else
-        object.nodes.count
-      end
-    end
-  end
-
-  class CustomBaseEdge < GraphQL::Relay::Edge
-    def upcased_name
-      node.name.upcase
-    end
-
-    def upcased_parent_name
-      parent.name.upcase
+      object.items.count
     end
   end
 
@@ -78,9 +63,9 @@ module StarWars
 
   class CustomBaseEdgeType < GraphQL::Types::Relay::BaseEdge
     node_type(BaseType)
-    field :upcased_name, String, null: true
-    field :upcased_parent_name, String, null: true
-    field :edge_class_name, String, null: true
+    field :upcased_name, String
+    field :upcased_parent_name, String
+    field :edge_class_name, String
 
     def edge_class_name
       object.class.name
@@ -88,19 +73,13 @@ module StarWars
   end
 
   class CustomEdgeBaseConnectionType < GraphQL::Types::Relay::BaseConnection
-    edge_type(CustomBaseEdgeType, edge_class: (TESTING_INTERPRETER ? NewCustomBaseEdge : CustomBaseEdge), nodes_field: true)
-    field :total_count_times_100, Integer, null: true
+    edge_type(CustomBaseEdgeType, edge_class: NewCustomBaseEdge, nodes_field: true)
+    field :total_count_times_100, Integer
     def total_count_times_100
-      if object.is_a?(GraphQL::Pagination::Connection)
-        # new-style
-        object.items.count * 100
-      else
-        # legacy
-        object.nodes.count * 100
-      end
+      object.items.count * 100
     end
 
-    field :field_name, String, null: true
+    field :field_name, String
     def field_name
       object.field.name
     end
@@ -144,7 +123,7 @@ module StarWars
       GraphQL::Relay::GlobalIdResolve.new(type: Faction).call(object, {}, context)
     end
 
-    field :name, String, null: true
+    field :name, String
     field :ships, ShipConnectionWithParentType, connection: true, max_page_size: 1000, null: true do
       argument :name_includes, String, required: false
     end
@@ -177,7 +156,7 @@ module StarWars
 
     field :shipsWithMaxPageSize, "Ships with max page size", max_page_size: 2, resolver: ShipsWithMaxPageSize
 
-    field :bases, BasesConnectionWithTotalCountType, null: true, connection: true do
+    field :bases, BasesConnectionWithTotalCountType, connection: true do
       argument :name_includes, String, required: false
       argument :complex_order, Boolean, required: false
     end
@@ -193,8 +172,8 @@ module StarWars
       all_bases
     end
 
-    field :bases_clone, BaseConnection, null: true
-    field :bases_by_name, BaseConnection, null: true do
+    field :bases_clone, BaseConnection
+    field :bases_by_name, BaseConnection do
       argument :order, String, default_value: "name", required: false
     end
     def bases_by_name(order: nil)
@@ -213,14 +192,14 @@ module StarWars
       all_bases.to_a
     end
 
-    field :basesWithMaxLimitRelation, BaseConnection, null: true, max_page_size: 2, resolver_method: :all_bases
-    field :basesWithMaxLimitArray, BaseConnection, null: true, max_page_size: 2, resolver_method: :all_bases_array
-    field :basesWithDefaultMaxLimitRelation, BaseConnection, null: true, resolver_method: :all_bases
-    field :basesWithDefaultMaxLimitArray, BaseConnection, null: true, resolver_method: :all_bases_array
-    field :basesWithLargeMaxLimitRelation, BaseConnection, null: true, max_page_size: 1000, resolver_method: :all_bases
-    field :basesWithoutNodes, BaseConnectionWithoutNodes, null: true, resolver_method: :all_bases_array
+    field :basesWithMaxLimitRelation, BaseConnection, max_page_size: 2, resolver_method: :all_bases
+    field :basesWithMaxLimitArray, BaseConnection, max_page_size: 2, resolver_method: :all_bases_array
+    field :basesWithDefaultMaxLimitRelation, BaseConnection, resolver_method: :all_bases
+    field :basesWithDefaultMaxLimitArray, BaseConnection, resolver_method: :all_bases_array
+    field :basesWithLargeMaxLimitRelation, BaseConnection, max_page_size: 1000, resolver_method: :all_bases
+    field :basesWithoutNodes, BaseConnectionWithoutNodes, resolver_method: :all_bases_array
 
-    field :bases_as_sequel_dataset, BasesConnectionWithTotalCountType, null: true, connection: true, max_page_size: 1000 do
+    field :bases_as_sequel_dataset, BasesConnectionWithTotalCountType, connection: true, max_page_size: 1000 do
       argument :name_includes, String, required: false
     end
 
@@ -232,7 +211,7 @@ module StarWars
       all_bases
     end
 
-    field :basesWithCustomEdge, CustomEdgeBaseConnectionType, null: true, connection: true, resolver_method: :lazy_bases
+    field :basesWithCustomEdge, CustomEdgeBaseConnectionType, connection: true, resolver_method: :lazy_bases
 
     def lazy_bases
       LazyNodesWrapper.new(object.bases)
@@ -244,11 +223,11 @@ module StarWars
 
     # Nested under `input` in the query:
     argument :ship_name, String, required: false
-    argument :faction_id, ID, required: true
+    argument :faction_id, ID
 
     # Result may have access to these fields:
-    field :ship_edge, Ship.edge_type, null: true
-    field :faction, Faction, null: true
+    field :ship_edge, Ship.edge_type
+    field :faction, Faction
     field :aliased_faction, Faction, hash_key: :aliased_faction, null: true
 
     def resolve(ship_name: nil, faction_id:)
@@ -261,13 +240,16 @@ module StarWars
       else
         ship = DATA.create_ship(ship_name, faction_id)
         faction = DATA["Faction"][faction_id]
-        connection_class = GraphQL::Relay::BaseConnection.connection_for_nodes(faction.ships)
-        ships_connection = connection_class.new(faction.ships, {ship_name: ship_name, faction: faction})
-        ship_edge = GraphQL::Relay::Edge.new(ship, ships_connection)
+        range_add = GraphQL::Relay::RangeAdd.new(
+          collection: faction.ships,
+          item: ship,
+          parent: faction,
+          context: context,
+        )
         result = {
-          ship_edge: ship_edge, # support new-style, too
-          faction: faction,
-          aliased_faction: faction,
+          ship_edge: range_add.edge,
+          faction: range_add.parent,
+          aliased_faction: range_add.parent,
         }
         if ship_name == "Slave II"
           LazyWrapper.new(result)
@@ -324,15 +306,6 @@ module StarWars
   end
 
   LazyNodesWrapper = Struct.new(:relation)
-  class LazyNodesRelationConnection < GraphQL::Relay::RelationConnection
-    def initialize(wrapper, *args, **kwargs)
-      super(wrapper.relation, *args, **kwargs)
-    end
-
-    def edge_nodes
-      LazyWrapper.new { super }
-    end
-  end
 
   class NewLazyNodesRelationConnection < GraphQL::Pagination::ActiveRecordRelationConnection
     def initialize(wrapper, **kwargs)
@@ -344,28 +317,26 @@ module StarWars
     end
   end
 
-  GraphQL::Relay::BaseConnection.register_connection_implementation(LazyNodesWrapper, LazyNodesRelationConnection)
-
   class QueryType < GraphQL::Schema::Object
     graphql_name "Query"
 
-    field :rebels, Faction, null: true
+    field :rebels, Faction
     def rebels
       StarWars::DATA["Faction"]["1"]
     end
 
-    field :empire, Faction, null: true
+    field :empire, Faction
     def empire
       StarWars::DATA["Faction"]["2"]
     end
 
-    field :largest_base, BaseType, null: true
+    field :largest_base, BaseType
 
     def largest_base
       Base.find(3)
     end
 
-    field :newest_bases_grouped_by_faction, BaseConnection, null: true
+    field :newest_bases_grouped_by_faction, BaseConnection
 
     def newest_bases_grouped_by_faction
       Base
@@ -380,47 +351,27 @@ module StarWars
       [OpenStruct.new(id: nil)]
     end
 
-    if TESTING_INTERPRETER
-      add_field(GraphQL::Types::Relay::NodeField)
-    else
-      field :node, field: GraphQL::Relay::Node.field
+    include GraphQL::Types::Relay::HasNodeField
+
+    field :node_with_custom_resolver, GraphQL::Types::Relay::Node do
+      argument :id, ID
+    end
+    def node_with_custom_resolver(id:)
+      StarWars::DATA["Faction"]["1"]
     end
 
-    if TESTING_INTERPRETER
-      field :node_with_custom_resolver, GraphQL::Types::Relay::Node, null: true do
-        argument :id, ID, required: true
-      end
-      def node_with_custom_resolver(id:)
-        StarWars::DATA["Faction"]["1"]
-      end
-    else
-      custom_node_field = GraphQL::Relay::Node.field do
-        resolve ->(_, _, _) { StarWars::DATA["Faction"]["1"] }
-      end
-      field :nodeWithCustomResolver, field: custom_node_field
+
+    include GraphQL::Types::Relay::HasNodesField
+
+    field :nodes_with_custom_resolver, [GraphQL::Types::Relay::Node, null: true] do
+      argument :ids, [ID]
+    end
+    def nodes_with_custom_resolver(ids:)
+      [StarWars::DATA["Faction"]["1"], StarWars::DATA["Faction"]["2"]]
     end
 
-    if TESTING_INTERPRETER
-      add_field(GraphQL::Types::Relay::NodesField)
-    else
-      field :nodes, field: GraphQL::Relay::Node.plural_field
-    end
-
-    if TESTING_INTERPRETER
-      field :nodes_with_custom_resolver, [GraphQL::Types::Relay::Node, null: true], null: true do
-        argument :ids, [ID], required: true
-      end
-      def nodes_with_custom_resolver(ids:)
-        [StarWars::DATA["Faction"]["1"], StarWars::DATA["Faction"]["2"]]
-      end
-    else
-      field :nodesWithCustomResolver, field: GraphQL::Relay::Node.plural_field(
-        resolve: ->(_, _, _) { [StarWars::DATA["Faction"]["1"], StarWars::DATA["Faction"]["2"]] }
-      )
-    end
-
-    field :batched_base, BaseType, null: true do
-      argument :id, ID, required: true
+    field :batched_base, BaseType do
+      argument :id, ID
     end
 
     def batched_base(id:)
@@ -433,37 +384,12 @@ module StarWars
     field :introduceShip, mutation: IntroduceShipMutation
   end
 
-  class ClassNameRecorder
-    def initialize(context_key)
-      @context_key = context_key
-    end
-
-    def instrument(type, field)
-      inner_resolve = field.resolve_proc
-      key = @context_key
-      field.redefine {
-        resolve ->(o, a, c) {
-          res = inner_resolve.call(o, a, c)
-          if c[key]
-            c[key] << res.class.name
-          end
-          res
-        }
-      }
-    end
-  end
-
   class Schema < GraphQL::Schema
     query(QueryType)
     mutation(MutationType)
     default_max_page_size 3
 
-    if TESTING_INTERPRETER
-      use GraphQL::Execution::Interpreter
-      use GraphQL::Analysis::AST
-      use GraphQL::Pagination::Connections
-      connections.add(LazyNodesWrapper, NewLazyNodesRelationConnection)
-    end
+    connections.add(LazyNodesWrapper, NewLazyNodesRelationConnection)
 
     def self.resolve_type(type, object, ctx)
       if object == :test_error
@@ -485,13 +411,13 @@ module StarWars
     end
 
     def self.id_from_object(object, type, ctx)
+      if !type.is_a?(Class)
+        raise "This handler should always receive a new-style class definition"
+      end
       GraphQL::Schema::UniqueWithinType.encode(type.graphql_name, object.id)
     end
 
     lazy_resolve(LazyWrapper, :value)
     lazy_resolve(LazyLoader, :value)
-
-    instrument(:field, ClassNameRecorder.new(:before_built_ins))
-    instrument(:field, ClassNameRecorder.new(:after_built_ins), after_built_ins: true)
   end
 end
